// algebra.h v 1.0 09.2003 (mGC Library)
// Definition of pZ2 pR3 vR3 m33 classes
// Copyright (C) 2003 L.E.R.I.  All rights reserved.
// Reims University (FRANCE)

#ifndef __ALGEBRA__
#define __ALGEBRA__

#include <iostream>
#include <cmath>

#include <define.h>
#include <color.h>

namespace mCG
{

class pN2
{
	public:

	long x,y;

	pN2(long a=0,long b=0);
	pN2(const pN2& p);

	inline pN2& operator = (const pN2& p)
    {
	   x=p.x , y=p.y;
	   return (*this);
    }

    inline pN2 operator - () const
    {	return pN2(-x,-y); }

	inline bool operator == (const pN2 &p) const
    { return x==p.x && y==p.y ;  }

    inline bool operator != (const pN2 &p) const
    {	return !((*this)==p) ;}

    inline bool operator < (const pN2 &p) const
    {	return y<p.y;}

    inline bool operator <= (const pN2 &p) const
    {	return y<=p.y;}

    inline bool operator > (const pN2 &p) const
    {	return y>p.y;}

    inline bool operator >= (const pN2 &p) const
    {	return y>=p.y;}

	
	inline pN2  absolue()
    {	return ( (x && y )<0 ? pN2(-x,-y) : ( x<0 ? pN2(-x,y) : (y<0 ? pN2(x,-y) : pN2(x,y)))); }

};

class vR3;

class pR3
{
	public:

	double x, y, z; // cartesian point coordinate

	// construction
	pR3(double _x=0., double _y=0., double _z=0.);
	pR3(const vR3 &v);

	// arithmetical operators
	pR3 & operator += (const vR3 &v);
	pR3 & operator *= (double k);
	pR3 & operator *= (const vR3 &v);
	pR3 & operator /= (double k);
	pR3 & operator /= (const vR3 &v);
	pR3 operator + (const vR3 &v) const;
    vR3 operator - (const pR3 &p) const;
    pR3 operator * (const vR3 &v) const;
    pR3 operator / (const vR3 &v) const;
    
    inline pR3 operator - () const
    {	return pR3(-x,-y,-z);}

    inline pR3 operator * (double k) const
    {	return pR3(x*k, y*k, z*k);}
    
    inline pR3 operator / (double k) const
    {
         assert(k > EPS);
         return pR3(x/k, y/k, z/k);
    }
	
	// logical operators
    inline bool operator == (const pR3 &p) const
    {	return fabs(x-p.x)<=EPS && fabs(y-p.y)<=EPS && fabs(z-p.z)<=EPS;}

    inline bool operator != (const pR3 &p) const
    {	return !(*this == p);}

    inline bool operator > (const pR3 &p) const
    {	return !(*this <= p);}

    inline bool operator >= (const pR3 &p) const
    {	return !(*this < p);}

    inline bool operator < (const pR3 &p) const
    {	return x<p.x && y<p.y && z<p.z;}

    inline bool operator <= (const pR3 &p) const
    {	return *this<p || *this==p;}



	// others operators
	inline double operator [] (int i) const
    {
        assert(0 <= i && i < 3);
        return i==0?x:(i==1?y:z);
    }

    inline double& operator [] (int i)
    {
       assert(0 <= i && i < 3);
       return i==0?x:(i==1?y:z);
    }


	// friendly functions
	//friend void FromPolar (double theta, double phi, double rho, pR3 &p);

	friend std::ostream & operator << (std::ostream &o,const pR3 &p);
	
	// special point
	static const pR3 ZERO;
};

class pR4
{
	public:
		pR3 coords;
		double homogen;
		
		pR4(const pR3& p=pR3());
		pR4(const double&, const double&, const double&);
		pR4(const double&, const double&, const double&, const double&);
		pR4& operator =(const pR3&);
		pR4& operator =(const pR4&);
};

///////////////////////VR3//////////////////////////////

class vR3
{
	public:
	
   double x, y, z; // cartesian vector components
	
	// construction
	vR3(double _x=0.,double _y=0.,double _z=0.);
	vR3(const pR3 &p); 
	vR3(const pR3 &p1, const pR3 &p2);

	// arithmetical operators
	vR3   operator +  (const vR3 &v) const;
	vR3 & operator += (const vR3 &v);
	vR3   operator -  () const;
	vR3   operator -  (const vR3 &v) const;
	vR3 & operator -= (const vR3 &v);
	vR3   operator *  (double k) const;
	vR3 & operator *= (double k);
	vR3   operator /  (double k) const;
	vR3 & operator /= (double k);
	
	// logical operators
	bool  operator == (const vR3 &v) const;
	bool  operator != (const vR3 &v) const;
	bool  operator >  (const vR3 &v) const;
	bool  operator >= (const vR3 &v) const;
	bool  operator <  (const vR3 &v) const;
	bool  operator <= (const vR3 &v) const;

	// others operators
	double   operator [] (int i) const;
	double & operator [] (int i);
	bool     operator || (const vR3 &v) const;  // parallelism

	// methods
	double operator ! (void) const;          // vector magnitude
	vR3 & Normalize ();
	vR3   Normalize () const;                      // vector normalization

	double operator % (const vR3 &v) const;  // dot product
	vR3    operator ^ (const vR3 &v) const;  // cross product
	double Dot        (const vR3 &v) const;
	vR3    Cross      (const vR3 &v) const;
	vR3    UnitCross  (const vR3 &v) const;

   // friendly functions	
	friend vR3    Normalize      (const vR3 &v);
	friend double MixedProduct   (const vR3 & v1, const vR3 &v2, const vR3 &v3);
	friend void   Orthonormalize (vR3 &u, vR3 &v, vR3 &w);

	friend std::ostream & operator <<  (std::ostream &o,const vR3 &v);

	// special vectors
	static const vR3 NUL;
	static const vR3 UNIT_X;
	static const vR3 UNIT_Y;
	static const vR3 UNIT_Z;
};



class m33
{
	public:

	vR3 vx, vy, vz; // vector matrix components

	// construction
	m33(double a =1.,double b =0.,double c =0.,
	    double d =0.,double e =1.,double f =0.,
	    double g =0.,double h =0.,double i =1.,bool _column = true); 
	m33(const vR3 &_x,const vR3 &_y,const vR3 &_z);

	m33   operator +  (const m33 &m) const;
	m33 & operator += (const m33 &m);
	m33   operator -  (const m33 &m) const;
	m33 & operator -= (const m33 &m);
	m33   operator *  (double k) const;
	m33 & operator *= (double k); 
	pR3   operator *  (const pR3 &p) const;
	vR3   operator *  (const vR3 &v) const;
	m33   operator *  (const m33 &m) const;
	m33 & operator *= (const m33 &m);

	double operator ! (void) const;          // return de square of the Norm

	double Det () const; // determinant
	m33    Inv () const; // inverse
	m33    Trs () const; // transpose

   // friendly functions	
	friend std::ostream & operator << (std::ostream &o,const m33 &m);

	// special matrices
	static const m33 ZERO;
	static const m33 IDENTITY;
};

class Transform
{
	public:

	vR3 vx,vy,vz,vt;
	pR4 px, py, pz, ph;

	Transform(double a =1.,double b =0.,double c =0.,
	         double d =0.,double e =1.,double f =0.,
				double g =0.,double h =0.,double i =1.,
				double t1=0.,double t2=0.,double t3=0.);
	
	Transform(double x1, double x2, double x3, double x4, 
		  double y1, double y2, double y3, double y4,
		  double z1, double z2, double z3, double z4,
		  double h1, double h2, double h3, double h4);
	
	Transform(const vR3 &x, const vR3 &y, const vR3 &z, const vR3 &t);

	Transform   operator +  (const Transform& m) const;
	Transform & operator += (const Transform& m);
	Transform   operator -  (const Transform& m) const;
	Transform & operator -= (const Transform& m);
	Transform   operator *  (const double r) const;
	Transform & operator *= (const double r); 
	vR3         operator *  (const vR3& v) const;
	pR3         operator *  (const pR3& p) const;
	pR4	    operator *(const pR4&) const; 
	Transform   operator *  (const Transform& m) const;
	Transform & operator *= (const Transform& m);
	Transform   operator /  (const double r) const;
	Transform & operator /= (const double r); 

	double    det() const; // determinant
	Transform inv() const; // inverse

	// fonctions amies

	friend std::ostream & operator << (std::ostream &o,const Transform &m);
};

class CRep
{
	public:

	Transform mat, invMat;

	CRep();
	CRep(const vR3 &v1,const vR3 &v2,const vR3 &v3,const vR3 &v4);
	CRep(const Transform &A);

	CRep inv () const;

	virtual CRep   operator *  (const CRep &Tr) const;
	virtual CRep & operator *= (const CRep &Tr);

	virtual vR3 Apply    (const vR3 &v) const;
	virtual vR3 invApply (const vR3 &v) const;
	virtual pR3 Apply    (const pR3 &p) const;
	virtual pR3 invApply (const pR3 &p) const;

	// fonctions amies

	friend std::ostream & operator << (std::ostream &o,const CRep &cr);
};

class Trans : public CRep
{
	public:

	Trans(double a,double b,double c);
	Trans(const vR3 &v);

	CRep   operator *  (const CRep &Tr) const;
	CRep & operator *= (const CRep &Tr);

	vR3 Apply    (const vR3 &v) const;
	vR3 invApply (const vR3 &v) const;
	pR3 Apply    (const pR3 &p) const;
	pR3 invApply (const pR3 &p) const;
};

class RotX : public CRep
{
	public:

	RotX(double theta);
};

class RotY : public CRep
{
	public:

	RotY(double theta);
};

class RotZ : public CRep
{
	public:

	RotZ(double theta);
};

class RotA : public CRep
{
	public:

	RotA(double x,double y,double z,double theta);
	RotA(const vR3 &v,double theta);
};

class Scale : public CRep
{
	public:

	Scale(double x,double y,double z);
};

 // namespace mCG

class Sommet : public pR3
{
	protected :
		Color c;

	public :
		vR3 norm;

		Sommet(double x,double y,double z=0,Color
			couleur=Color(250.,250.,250.)) : pR3(x,y,z),c(couleur),norm(x,y,z)
		{
			norm.Normalize();
		}

		Sommet(pR3 point,Color couleur=Color(250.,250.,250.)) :
			pR3(point),c(couleur) {}

		Color setColor(Color nouv)
		{
			Color res(c);
			c=nouv;
			return res;
		}
		
		Color getColor() const
		{
			return c;
		}

		Sommet& operator = (const Sommet& s)
		{
			pR3::operator = (s);
			c=s.c;
			norm = s.norm;
			return *this;
		}

};

class Face
{
	public :
		unsigned index[3];

		Face()
		{
			(*index)   = 0;
			*(index+1) = 0;
			*(index+2) = 0;
		}
		
		Face(unsigned x, unsigned y, unsigned z)
		{
			(*index)   = x;
			*(index+1) = y;
			*(index+2) = z;
		}

		Face(const Face& _f)
		{
			memcpy(index, _f.index, sizeof(unsigned)*3);
		}
		
		Face& operator =(const Face& _f)
		{
			memcpy(index, _f.index, sizeof(unsigned)*3);
		}
		
};

struct Intervalle
{
	int a, b;
};

}
#endif  // __ALGEBRA__
