// color.h v 1.0 09.2003 (mGC Library)
// Definition of cN3 cR3 classes
// Copyright (C) 2003 L.E.R.I.  All rights reserved.
// Reims University (FRANCE)

#ifndef __COLOR__
#define __COLOR__

#include <define.h>

#include <iostream>
#include <cmath>


namespace mCG
{

const RGB RGB_MASK = 0x00ffffff; // masks RGB values

inline int getRed   (RGB rgb) { return (int)(rgb & 0xff);         }
inline int getGreen (RGB rgb) { return (int)((rgb >> 8) & 0xff);  }
inline int getBlue  (RGB rgb)	{ return (int)((rgb >> 16) & 0xff); }

inline int getGray (int r, int g, int b) { return (r*11+g*16+b*5)/32; }
inline int getGray (RGB rgb            ) { return getGray (getRed(rgb), getGreen(rgb), getBlue(rgb)); }
inline RGB getRGB  (int r, int g, int b) { return (RGB)(r&0xff)|((RGB)(g&0xff)<<8)|((RGB)(b&0xff)<<16); }

class cR3;

class cN3
{
	private:

	RGB value;

	public:

	enum TypeColor { Rgb, Hsv };

	// constructors
	cN3(int _r=250, int _g=250, int _b=250) { setRgb(_r,_g,_b); }
	cN3(int _x, int _y, int _z, TypeColor tc);
	cN3(const cR3 &c);

	RGB rgb   () const { return value & RGB_MASK; }
	int red   () const { return getRed   (value); }
	int green () const { return getGreen (value); }
	int blue  () const { return getBlue  (value); }

	void rgb    (int *r, int *g, int *b) const;
	void hsv    (int *h, int *s, int *v) const;
	void setRgb (int r, int g, int b);
	void setHsv (int h, int s, int v);
	void setRgb (RGB rgb);

	cN3 Light (int factor = 150) const;
	cN3 Dark  (int factor = 200) const;

	// logical operators
	bool operator == (const cN3 &c) const;
	bool operator != (const cN3 &c) const;

	// others operators
	int   operator [] (int i) const;

   // friendly functions	
	friend std::ostream & operator << (std::ostream &, const cN3 &c);

	// special cN3s
	static const cN3 BLACK;
	static const cN3 WHITE;
	static const cN3 DARKGRAY;
	static const cN3 GRAY;
	static const cN3 LIGHTGRAY;
	static const cN3 RED;
	static const cN3 GREEN;
	static const cN3 BLUE;
	static const cN3 CYAN;
	static const cN3 MAGENTA;
	static const cN3 YELLOW;
	static const cN3 DARKRED;
	static const cN3 DARKGREEN;
	static const cN3 DARKBLUE;
	static const cN3 DARKCYAN;
	static const cN3 DARKMAGENTA;
	static const cN3 DARKYELLOW;
};

class cR3
{
	public:

	double r, g, b; // color definition

	// construction
	cR3(double _r=250, double _g=250, double _b=250);
	cR3(const cN3 &c);

	// arithmetical operators
	cR3   operator +  (const cR3 &c) const;
	cR3 & operator += (const cR3 &c);
	cR3   operator -  () const;
	cR3   operator -  (const cR3 &c) const;
	cR3 & operator -= (const cR3 &c);
	cR3   operator *  (double k) const;
	cR3 & operator *= (double k);
	cR3   operator *  (const cR3 &c) const;
	cR3 & operator *= (const cR3 &c);
	cR3   operator /  (double k) const;
	cR3 & operator /= (double k);
	cR3   operator /  (const cR3 &c) const;
	cR3 & operator /= (const cR3 &c);
	
	// logical operators
	bool  operator == (const cR3 &c) const;
	bool  operator != (const cR3 &c) const;

	// others operators
	double   operator [] (int i) const;
	double & operator [] (int i);

	// methods
	double operator ! (void) const; // color magnitude
	cR3 & Normalize ();             // color normalization
	void  Clamp();

	// friendly functions
	friend cR3 Normalize (const cR3 &c);

	friend std::ostream & operator << (std::ostream &o,const cR3 &c);
};

typedef struct cN3 Pixel;
typedef struct cR3 Color;

} // namespace mCG

#endif // __COLOR__
