// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */



#ifndef _LOADER_H_
#define _LOADER_H_

#include <iostream>
#include <fstream>
#include <vector>
#include <stdlib.h>

#include <motor/objet.hh>


// Specifics MD2 files define

#define MD2_IDENT               (('2'<<24) + ('P'<<16) + ('D'<<8) + 'I')
#define MD2_VERSION             8
#define NUMVERTEXNORMALS        162


using namespace std;


struct Md2Header
{
	int ident;          // numro magique : "IDP2"
    	int version;        // version du format : 8

	int skinwidth;      // largeur texture
	int skinheight;     // hauteur texture

	int framesize;      // taille d'une frame en octets

	int num_skins;      // nombre de skins
	int num_vertices;   // nombre de vertices par frame
	int num_st;         // nombre de coordonnes de texture
	int num_tris;       // nombre de triangles
	int num_glcmds;     // nombre de commandes opengl
	int num_frames;     // nombre de frames

	int offset_skins;   // offset donnes skins
	int offset_st;      // offset donnes coordonnes de texture
	int offset_tris;    // offset donnes triangles
	int offset_frames;  // offset donnes frames
	int offset_glcmds;  // offset donnes commandes OpenGL
	int offset_end;     // offset fin de fichier
};

struct vec3_t
{
	float x, y, z;
};

struct vertMd2
{
	unsigned char v[3];         // position dans l'espace (relative au modle)
    	unsigned char normalIndex;  // index normale du vertex
};

struct triangMd2
{
	unsigned short vertex[3];   // indices vertices du triangle
    	unsigned short st[3];       // indices coordonnes de texture
};

struct texMd2
{
	short s;
    	short t;
};

struct frameMd2
{
	vec3_t          scale;      // redimensionnement
    	vec3_t          translate;  // vecteur translation
    	char            name[16];   // nom de la frame
    	vertMd2    	*verts; 
};

struct skinMd2
{
	char    name[68];
};

class loader
{
	protected:
		SPolyhedron** obj;
	
	public:
		virtual ~loader() {}
		virtual bool load(const char* fic) = 0;
};

class loaderPLY : public loader
{
	public:
		loaderPLY(SPolyhedron**);
		~loaderPLY();
	
		bool load(const char*);
};

class SEntity;

class loaderMAX : protected loader
{
	private:
		void readMaterials(ifstream&, long, vector<Color>& lColor);
	protected:
		SEntity** enti;
	public:
		loaderMAX(SEntity**);
		~loaderMAX();
	
		bool load(const char*);
};

class Md2Model
{
	friend class SEntity;
	friend class SMonstre;
	
	protected:
	
		Md2Header	m_kHeader;      // header md2
   		skinMd2	      	*m_pSkins;      // donnes skins
    		texMd2		*m_pTexCoords;  // coordonnes de texture
    		triangMd2	*m_pTriangles;  // donnes triangles
    		frameMd2	*m_pFrames;     // donnes frames
    		int             *m_pGLcmds;     // liste de commandes OpenGL

    		GLfloat         m_fScale;       // redimensionnement du modle
		vec3_t   	m_kAnorms[NUMVERTEXNORMALS];
		
	public:
		
	
		Md2Model();
		~Md2Model();
		
		bool loadModel(const char*);
		void setNormals(const char*);
		void display();
};



#endif // _LOADER_H_
