// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#ifndef __CAMERA_HH__
#define __CAMERA_HH__


#include <cmath>
#include <GL/glut.h>

#include <algebra.h>
#include <color.h>
#include <motor/objet.hh>

using namespace mCG;

class Camera
{
	friend class HUD;
  protected: 
  
    int x,   // last x coord from MotionFunc
	y,   // last y coord from MotionFunc
	acl; // Acceleration
    vR3 Verticale;
    float Roty; 
    
    vR3 RotY( const vR3& v, float angle )
    {
      return vR3(v.x*cos(angle)-v.z*sin(angle),v.y,v.x*sin(angle)+v.z*cos(angle));
    }
    
    vR3 RotX( const vR3& v, float angle )
    {
      return vR3(v.x,v.y*cos(angle)+v.z*sin(angle),-v.y*sin(angle)+v.z*cos(angle));
    } 
    
  public:
  
    pR3 Oeil;
    vR3 Viser, ViserZ;
  
    Camera() : Oeil(0.,-25.25,10.), Viser(0.,0.,-1.), Verticale(0.,1.,0.),
	       ViserZ(0.,0.,-1.), x(-1), y(-1), Roty(0.), acl(1) {}
    Camera( 	  GLdouble eyex,
		  GLdouble eyey,
		  GLdouble eyez,
		  GLdouble centerx,
		  GLdouble centery,
		  GLdouble centerz,
		  GLdouble upx,
		  GLdouble upy,
		  GLdouble upz ) : Oeil(eyex, eyey, eyez),
		                   Viser(centerx, centery, centerz),
				   Verticale(upx, upy, upz), ViserZ(0.,0.,-1.), x(-1), y(-1), Roty(0.), acl(10)
    {}
    
    void Display() { gluLookAt( Oeil.x, Oeil.y, Oeil.z,
       			        Oeil.x+Viser.x, Oeil.y+Viser.y, Oeil.z+Viser.z, Verticale.x, Verticale.y, Verticale.z
			      );
    }
   
   void Mouse(int button, int state, int _x, int _y)
   {
     x = _x;
     y = _y;
   }
    
   void Motion(int _x, int _y)
   {
        vR3 tmp;
	float n(_x-x);
	
        if( n )
	{
	  Viser = RotY(Viser,(n = (n / 300.))).Normalize();
	  Roty += n;
	 /* if( 360 < Roty )
	    Roty -= 360;
	  else if( 360 < -Roty )
	         Roty += 360;*/
	 
	}
	
	if( (n=(_y-y)) ) 
	{
	  tmp = RotX(ViserZ, n/300.).Normalize();
	  if( tmp.z < 0. )
	  {
	  	ViserZ = tmp;
		Viser = RotY(ViserZ,Roty).Normalize();
	  }  
	}
	x = _x;
	y = _y;
   }
   
    
   void Keyboard( unsigned int key, int x, int y )
   {
      vR3 tmp(RotY(Viser, M_PI/2.).Normalize()/2.5);
      switch (key) {
        case '8':
	  Oeil.x += Viser.x/(float)0.5;
	  Oeil.z += Viser.z/(float)0.5;
	break;
	case '2':
	  Oeil.x -= Viser.x/(float)0.25;
	  Oeil.z -= Viser.z/(float)0.5;
	break;
        case '4':
	  Oeil.x -= tmp.x/(float)0.5;
	  Oeil.z -= tmp.z/(float)0.5;
	break;
	case '6':
	  Oeil.x += tmp.x/0.5;
	  Oeil.z += tmp.z/0.5;
	break;
        default : break;      
      }  
   }


   void Special( int key, int x, int y )
   {
      vR3 tmp(RotY(Viser, M_PI/2.).Normalize()/5.);
      switch (key) {
        case GLUT_KEY_UP:
	  Oeil.x += Viser.x/5.;
	  Oeil.z += Viser.z/5.;
	break;
	case GLUT_KEY_DOWN:
	  Oeil.x -= Viser.x/5.;
	  Oeil.z -= Viser.z/5.;
	break;
        case GLUT_KEY_LEFT:
	  Oeil.x -= tmp.x;
	  Oeil.z -= tmp.z;
	break;
	case GLUT_KEY_RIGHT:
	  Oeil.x += tmp.x;
	  Oeil.z += tmp.z;
	break;
        default : break;      
      }  
   }
   
   void keyPressed(  ) { if( acl > 2 ) acl--; }
   
   void keyNotPressed(  ) { if( acl < 12 ) acl++; }
   

   float getAngleY() { return ( Viser.x > 0 ) ? acos(vR3(Viser.x, 0., Viser.z)%vR3(0., 0., -1.)) : -acos(vR3(Viser.x, 0., Viser.z)%vR3(0., 0., -1.)); }  
   
};

class CameraFP :public  Camera
{
	public:
		SBox*  box;
		
		CameraFP();
		CameraFP(const CameraFP&);
		~CameraFP();
		
		CameraFP& operator =(const CameraFP&);
		
		void Display();
  		void Motion(int _x, int _y);
		void Keyboard( unsigned int key, int x, int y );
		void KeyboardInv( unsigned int key, int x, int y );	
};

#endif //  __CAMERA_HH__
