// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */
 
 
 
#ifndef __REAL_OBJ_H__
#define __REAL_OBJ_H__

#include <motor/objet.hh>
#include <motor/texture.hh>
#include <motor/material.hh>
#include <motor/model.hh>

//#include "sarme.h"
using namespace mCG;

class SArme;

class SCaisse : public SBox
{
	protected:
		vR3 	   normale;
		TextureGL* tex;
	public:
		SCaisse();
		SCaisse(double _l, double _h, double _w, const pR3& _p=pR3(), TextureGL* tex_name=NULL, const vR3& _v=vR3());
		SCaisse(const SCaisse&);
		~SCaisse();
		
		SCaisse& operator =(const SCaisse&);
		
		bool loadTexture(TextureGL* tex_name);
		inline const vR3& getNormale() const
		{	return normale;	}

		//int getType() const;			// Inerited virtual method
		void display() const; 			// Inerited virtual method
};


class SMur : public SPlan
{
	protected:
		TextureGL* tex;
		pR3 gp_pt[4];
		Material	matiere;
		
		SMur(const pR3& _point0, const pR3& _point1, const pR3& _point2);
	
	public:
		SMur();
		SMur(const vR3& _normal, const double& _constant=0.);
    		SMur(const vR3& _normal, const pR3& _point);
    		SMur(const pR3& _point0, const pR3& _point1, const pR3& _point2, const pR3& _point3);
		SMur(const SMur&);
		
		~SMur();
		
		SMur& operator =(const SMur&);
		
		bool loadTexture(const char*);
		void display() const;
};


class SPerso : public SPolyhedron
{
	friend class loaderMAX;
	friend class SEntity;
	friend class SMonstre;
	
	protected:
		unsigned	nbpt;
		SBox*		boundBox;
		
		GLfloat*   	tab_vx;
		GLint*		tab_fc;
		GLfloat*	tab_tx;
		GLfloat*	tab_norm;
		TextureGL* 	tex;
		Material	matiere;
		
		// Private method
		
		void		calculNorm();
	
	public:
		unsigned	nbfc;
		SPerso(const double& _mass = 0., const pR3& _mCenter = pR3(), const vR3& _v0 = vR3(), const double& _density = 0.);
		SPerso(const SPerso&);
		~SPerso();
		
		SPerso& operator =(const SPerso&);
		
		bool loadTexture(const char*);
		bool loadModel(GLfloat&,GLfloat& , GLfloat&, GLfloat&, GLfloat& , GLfloat&);
		void display() const;
		inline void alloc(unsigned nsize)
		{
			if(tab_fc)
				delete[] tab_fc;
			tab_fc = new GLint[nsize];
			nbfc = nsize;
		}
		
		inline void set(GLint val, unsigned ind)
		{	tab_fc[ind] = val;}
		
		inline void allocTexture(unsigned nsize)
		{
			if(tab_tx)
				delete[] tab_tx;
			tab_tx = new GLfloat[nsize];
		}

		inline void setTexture(GLfloat val, unsigned ind)
		{	tab_tx[ind] = val;}

		inline SBox* boundingBox() const
		{	return boundBox;}

		inline int getType() const
		{	return S_PERSO_TYPE;}
		
		void setInitPos(const pR3&);

		inline void updatePosition(const pR3& p)
		{
			pR3 dpl(mass_center - p);
			mass_center = p;
			if(boundBox)
				boundBox->updatePosition(p);	
		}

		void resize(float);
		void rotateX(const double&);
		void rotateY(const double&);
		void rotateZ(const double&);
		
		inline void setAmbiantMaterial(const Color& col)
		{	matiere.setAmbiant(col);}

		inline void setDiffuseMaterial(const Color& col)
		{	matiere.setDiffuse(col);}

		inline void setSpecularMaterial(const Color& col)
		{	matiere.setSpecular(col);}

		
		void applyTransform(const Transform&, bool maj_norm=true);
};

class Model;
class Md2Model;

class SEntity : public SObject
{
	friend class loaderMAX;
	friend class SPerso;
	friend class SObject;
	friend class SInteraction;
	friend class Scene;
	
	protected:
		vR3		lookAt;
		SBox* 		boundBox;
		list<SPerso*>	allObj;
		Model* 		my_mod;
		
	public:
		SEntity();
		SEntity(Model&);
		SEntity(const SEntity&);
		~SEntity();

		//bool loadModel(const char*);
		bool loadFromMyModel(Model& );
	

		void setInitPos(const pR3&);
		void updatePosition(const pR3&);
		void resize(float);
		
		
		inline void rotateX(const double& angle)
		{
			for(list<SPerso*>::iterator itobj=allObj.begin();itobj!=allObj.end();itobj++)
				(*itobj)->rotateX(angle);
			boundBox->rotateX(angle);
			mass_center = boundBox->getMassCenter();
		}
		
		inline void rotateY(const double& angle)
		{
			for(list<SPerso*>::iterator itobj=allObj.begin();itobj!=allObj.end();itobj++)
				(*itobj)->rotateY(angle);
			boundBox->rotateY(angle);
			mass_center = boundBox->getMassCenter();
		}

		inline void rotateZ(const double& angle)
		{
			for(list<SPerso*>::iterator itobj=allObj.begin();itobj!=allObj.end();itobj++)
				(*itobj)->rotateZ(angle);
			boundBox->rotateZ(angle);
			mass_center = boundBox->getMassCenter();
		}

		
		inline void setSupport(SSupport& _s) 		// Inutile pr le moment (unused)
		{
			sSup = &_s;
			boundBox->setSupport(_s);
		}
		
		inline int getType() const
		{	return S_ENTITY_TYPE;	}

		inline SBox* boundingBox() const
		{	return boundBox;	}

		inline void display() const
		{
			if(sVisible)
				for(list<SPerso*>::const_iterator itobj=allObj.begin();itobj!=allObj.end();itobj++)
					(*itobj)->display();
		}
		
		void setAmbiantMaterial(int ind, const Color&);
		void setDiffuseMaterial(int ind, const Color&);
		void setSpecularMaterial(int ind, const Color&);

		inline void loadTexture(int ind, const char* tex_name)
		{
			list<SPerso*>::iterator it = allObj.begin();
			
			
			for(int i(0);i<ind;i++, it++);
				(*it)->loadTexture(tex_name);
		}
	
	
		// Permet le controle de la "bounding box" (boite englobante)
		
		inline void setV0(const vR3& _v)
		{
			SObject::setV0(_v);
			for(list<SPerso*>::iterator it = allObj.begin();it!= allObj.end(); it++)
				(*it)->setV0(_v);
			boundBox->setV0(_v);
		}

		inline void setDensity(const double& _d)
		{
			SObject::setDensity(_d);
			boundBox->setDensity(_d);
		}

		inline void setMass(const double& _m)
		{
			SObject::setMass(_m);
			boundBox->setMass(_m);
		}

		inline void applyRotationX(const Intervalle& Inter, const double& angle)
		{
			list<SPerso*>::iterator it = allObj.begin();
			for(int i(0);i<allObj.size();i++, it++)
				if(i >= Inter.a && i <= Inter.b)
					(*it)->SPerso::rotateX(angle);
		}

		inline void applyRotationY(const Intervalle& Inter, const double& angle)
		{
			list<SPerso*>::iterator it = allObj.begin();
			for(int i(0);i<allObj.size();i++, it++)
				if(i >= Inter.a && i <= Inter.b)
					(*it)->SPerso::rotateY(angle);
		}

		inline void applyRotationZ(const Intervalle& Inter, const double& angle)
		{
			list<SPerso*>::iterator it = allObj.begin();
			for(int i(0);i<allObj.size();i++, it++)
				if(i >= Inter.a && i <= Inter.b)
					(*it)->SPerso::rotateZ(angle);
		}
		
		// Application de transformations (oprations matricielles)
		
		void applyTransform(const Transform&, bool maj_norm=true);
		
		// DEBUG
		
		inline void displayObjNum(unsigned num) const
		{
			list<SPerso*>::const_iterator it = allObj.begin();
			for(int i(0);i!=num;i++, it++);
				(*it)->display();
		}

		// Accesseur(s)
		
		inline const vR3& getLookAt() const
		{	return lookAt; }

		
		// Setters
		
		inline void setLookAt(const vR3& _lookat)
		{	lookAt = _lookat;	}
};

class SMonstre: public SEntity
{
	public:
		SMonstre();
		SMonstre(Model&);
		~SMonstre();
		inline SEntity** getEntityPt()	{ return &entity_pt; }
		
		void Touched(float degat);
		void importMd2(Md2Model*, const char* fic_name=NULL);
		bool IsAlive();
		void display() const;
		void loadTexture(TextureGL*);
		
	private:
		float life,armor;
		SArme*	objActif;
		SEntity* entity_pt;
		Md2Model* md2Mod;
		TextureGL* tex;
};


#endif // __REAL_OBJ_H__
