// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */



#ifndef _SCENE_H_
#define _SCENE_H_

#include <AL/alut.h>
#include <GL/glut.h>
#include <iostream>
#include <stdlib.h>
#include <list>

#include <loader.hh>
#include <hud.hh>
#include <motor/objet.hh>
#include <motor/interaction.hh>
#include <motor/lumiere.hh>
#include <motor/camera.hh>
#include <motor/collision.hh>
#include <motor/ai.hh>
#include <motor/particle.hh>
#include <motor/model.hh>
#include <motor/allbullet.hh>
#include <motor/animation.hh>
#include <motor/sound.hh>

#include <all_init.h> //definition et fonction d'initialisation pour
			// les armes, models et textures

/***** Definition d'une classe scne permaettant :
	- Initialisation GL
	- Gestion de la scene: Murs, objets divers, interactions et camera(s)
	- Gestion des vnement (GLUT)
	
*****/

using namespace std;
using namespace mCG;

#define ALLE 	0
#define RETOUR 	1


class Scene
{
	protected:
		bool			keyIsPress;		// Camera & Mouse manager (character moving)
		CameraFP*		camera_main;
		HUD*			gui;			// HUD ==> User
		AllBullet* 	   	lBalleAllie;
		AllBullet*		lBalleEnemy;
		list<Lumiere*> 		lLum;			// All lights of scene
		list<SObject*>		lSol;			// The ground
		list<SObject*> 		lObj;			// Scene object(s) (not entity neither monster)
		list<SEntity*> 		lEnt;
		list<SArme*>		lArm2Pick;
		list<SArme*>		lArm;
		list<SInteraction*>	lInt;
		list<Animation*>	lAnim;
		SInteraction*		interact_cam;
		SInteraction*		interact_monstre;
		SMonstre* 		monstre;
		AI*			intel;
		
		Sound_Table *ttt;
		Sound *dddd;
		
	
		Scene(const Scene&);			// We don't have to copy a scene ==> unicity
		Scene& operator =(const Scene&);	// Idem
	
	public:
		Scene();
		~Scene();
		
		void initGL();
		void update();
		void addLumiere(Lumiere*);
		void addSol(SObject*);
		void addObject(SObject*, bool interact=false);
		void addEntity(SEntity*, bool interact=false);
		void addArme2Pick(SArme*);
		void addArme(SArme*, bool interact=false);
		void addMonstre(SMonstre*);
		void display();
		void mouseMoveManager(int button, int state, int x, int y);
		void mouseManager(int x, int y);
		void keyManager(unsigned char key, int x, int y, Model* gren_mod);
		
		void initInteract();
		void initLumiere();
		
		/*void init_model();
		void clean_model();
		void init_texture();
		void clean_texture();*/
		
};

#endif // _SCENE_H_
