// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */



#ifndef  __MYSOUND__
#define  __MYSOUND__

#include <AL/alut.h>

#include <algebra.h>

using namespace mCG;


static const int NB_SOUND=6;

enum SOUND_TYPE
{
	DOOR1,
	DOOR2,
	FOOT1,
	STEP2,
	GUN1,
	GUN2
};

static const char* SOUND_FILE[NB_SOUND] = {
	"Sound/Divres/Porte1.wav",
	"Sound/Divres/Porte2.wav",
	"Sound/Foot/FOOT.wav",
	"Sound/Foot/FOOT02.wav",
	"Sound/Gun/GUN01.wav",
	"Sound/Gun/GUN02.wav" };

static const char* SOUND_ERR[8] = {
		"GEN_SOURCE",
		"LOAD_SOUND",
		"UNLOAD_SOUND",
		"GET_BUFFER",
		"PLAY_SOUND",
		"STOP_SOUND",
		"SET_POS",
		"SET_VITESSE" };
		
		
class Sound_Exception
{
	friend class Sound_Table;
	friend class Sound;
	
private:
	int code;
	
public:

enum SOUND_EXCEPTION_CODE
{
	GEN_SOURCE=0,
	LOAD_SOUND=1,
	UNLOAD_SOUND=2,
	GET_BUFFER=3,
	PLAY_SOUND=4,
	STOP_SOUND=5,
	SET_POS=6,
	SET_VITESSE=7
};

	Sound_Exception(const SOUND_EXCEPTION_CODE&);
	~Sound_Exception();
	
	const int& getCode()const;
	const char* getChar()const;

};


class Sound_Table
{
	friend class Sound;
private:
	ALuint BUFFER[NB_SOUND];
	int compteur[NB_SOUND];
	
	void LoadSound(const SOUND_TYPE&) throw (Sound_Exception);
	void UnloadSound(const SOUND_TYPE&);
	
	const ALuint& getBuffer(const SOUND_TYPE&)const throw (Sound_Exception);

public:
	Sound_Table();
	~Sound_Table();
};


class Sound
{
private:
	SOUND_TYPE sound;
	ALuint source;
	Sound_Table *table;

public:
	Sound(const SOUND_TYPE&, Sound_Table*) throw (Sound_Exception);
	~Sound();
	
	void PlaySound(bool wait=true)const throw (Sound_Exception);
	void StopSound()const throw (Sound_Exception);
	
	void setPos(const pR3& )const throw (Sound_Exception);
	void setPos(const ALfloat&,const ALfloat&,const ALfloat& )const throw (Sound_Exception);
	void setVitesse(const pR3& )const throw (Sound_Exception);
	void setVitesse(const ALfloat&,const ALfloat&,const ALfloat& )const throw (Sound_Exception);
};




#endif 
