// algebra.cc v 1.0 09.2003 (mGC Library)
// Definition of pR3 vR3 m33 classes
// Copyright (C) 2003 L.E.R.I.  All rights reserved.
// Reims University (FRANCE)

#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include <cassert>


#include <algebra.h>

using namespace std;
using namespace mCG;

const pR3 pR3::ZERO    (0.0,0.0,0.0);
const vR3 vR3::NUL     (0.0,0.0,0.0);
const vR3 vR3::UNIT_X  (1.0,0.0,0.0);
const vR3 vR3::UNIT_Y  (0.0,1.0,0.0);
const vR3 vR3::UNIT_Z  (0.0,0.0,1.0);
const m33 m33::ZERO    (0.0,0.0,0.0,
                        0.0,0.0,0.0,
							   0.0,0.0,0.0);
const m33 m33::IDENTITY(1.0,0.0,0.0,
                        0.0,1.0,0.0,
								0.0,0.0,1.0);


//
// pZ2 interface
//

pN2::pN2(long a, long b):x(a),y(b) {}

pN2::pN2(const pN2& p):x(p.x),y(p.y) {}



//
// pR3 interface
//

pR3::pR3(double _x, double _y, double _z):x(_x),y(_y),z(_z){}

pR3::pR3(const vR3 &v):x(v.x),y(v.y),z(v.z){}

pR3 & pR3::operator += (const vR3 &v)
{
    x+=v.x; y+=v.y; z+=v.z;
    return *this;
}

pR3 & pR3::operator /= (double k)
{
   assert(k > EPS);
    x/=k; y/=k; z/=k;
    return *this;
}

pR3 & pR3::operator /= (const vR3 &v)
{
	assert(v.x > EPS && v.y > EPS && v.z > EPS);
    x/=v.x; y/=v.y; z/=v.z;
    return *this;
}


pR3 & pR3::operator *= (const vR3 &v)
{
   x*=v.x; y*=v.y; z*=v.z;
   return *this;
}

pR3 & pR3::operator *= (double k)
{
    x*=k; y*=k; z*=k;
    return *this;
}
pR3 pR3::operator + (const vR3 &v) const
{	return pR3(x+v.x, y+v.y, z+v.z); }

vR3 pR3::operator - (const pR3 &p) const
{	return vR3(x-p.x, y-p.y, z-p.z);}

pR3 pR3::operator * (const vR3 &v) const
{	return pR3(x*v.x, y*v.y, z*v.z);}

pR3 pR3::operator / (const vR3 &v) const
{
    assert(v.x > EPS && v.y > EPS && v.z > EPS);
    return pR3(x/v.x, y/v.y, z/v.z);
}

ostream & operator << (ostream &o, const pR3 &p)
{
	o << "p[" << p.x << "," << p.y << "," << p.z << "]" << endl;
	return o;
}

//
// vR3 interface
//

vR3::vR3(double _x, double _y, double _z):x(_x),y(_y),z(_z){}

vR3::vR3(const pR3 &p):x(p.x),y(p.y),z(p.z){}

vR3::vR3(const pR3 &p1, const pR3 &p2):x(p2.x-p1.x),y(p2.y-p1.y),z(p2.z-p1.z){}

vR3 vR3::operator + (const vR3 &v) const
{	return vR3(x+v.x, y+v.y, z+v.z); }

vR3 & vR3::operator += (const vR3 &v)
{
	x+=v.x;
    y+=v.y;
    z+=v.z;
	return *this;
}

vR3 vR3::operator - () const
{	return vR3(-x,-y,-z);}

vR3 vR3::operator - (const vR3 &v) const
{	return vR3(x-v.x, y-v.y, z-v.z);}

vR3 & vR3::operator -= (const vR3 &v)
{
	x-=v.x;
    y-=v.y;
    z-=v.z;
	return *this;
}

vR3 vR3::operator * (const double k) const
{	return vR3(x*k, y*k, z*k);}

vR3 & vR3::operator *= (const double k)
{
	x*=k;
    y*=k;
    z*=k;
	return *this;
}

vR3 vR3::operator / (const double k) const
{	//assert(k > EPS);
	return vR3(x/k, y/k, z/k);
}

vR3 & vR3::operator /= (const double k)
{	//assert(k > EPS);
	x/=k;
    y/=k;
    z/=k;
	return *this;
}

bool vR3::operator == (const vR3 &v) const
{	return fabs(x-v.x)<=EPS && fabs(y-v.y)<=EPS && fabs(z-v.z)<=EPS; }

bool vR3::operator != (const vR3 &v) const
{	return !(*this == v);}

bool vR3::operator > (const vR3 &v) const
{	return !(*this <= v);}

bool vR3::operator >= (const vR3 &v) const
{	return !(*this < v);}

bool vR3::operator < (const vR3 &v) const
{	return x<v.x && y<v.y && z<v.z; }

bool vR3::operator <= (const vR3 &v) const
{	return *this<v || *this==v;}

double vR3::operator [] (int i) const
{
    assert(0 <= i && i < 3);
    return i==0?x:(i==1?y:z);
}

double & vR3::operator [] (int i)
{
    assert(0 <= i && i < 3);
    return i==0?x:(i==1?y:z);
}

bool vR3::operator || (const vR3 &v) const
{ 	return (*this ^ v) == vR3::NUL; }

double vR3::operator ! (void) const
{	return SQ(x)+SQ(y)+SQ(z);}

vR3 & vR3::Normalize()
{
	double n(!(*this));
	//assert(n > EPS);
	if(n < EPS)
		n = EPS;
	*this /= sqrt(n);
	return *this;
}

vR3 vR3::Normalize() const
{
	double n(!(*this));
	//assert(n > EPS);
	if(n < EPS)
		n = EPS;
	vR3 res(*this / sqrt(n));
	return res;
}

double vR3::operator % (const vR3 &v) const
{	return x*v.x+y*v.y+z*v.z;}

vR3 vR3::operator ^ (const vR3 &v) const
{	return vR3(y*v.z-z*v.y,z*v.x-x*v.z,x*v.y-y*v.x);}

double vR3::Dot(const vR3 &v) const
{	return *this % v;}

vR3 vR3::Cross(const vR3 &v) const
{	return *this ^ v;}

vR3 vR3::UnitCross(const vR3 &v) const
{
	vR3 vcross(*this ^ v);
	return vcross.Normalize();
}

vR3 Normalize(const vR3 &v)
{
   	double n(!v);
	//assert(n > EPS);
	if(n < EPS)
		n = EPS;
	return v / sqrt(n);
}

double MixedProduct(const vR3 &v1, const vR3 &v2, const vR3 &v3)
{	return (v1^v2) % v3;}

void Orthonormalize(vR3 &u, vR3 &v, vR3 &w)
{
	// If the input vectors are v0, v1, and v2, then the Gram-Schmidt
	// orthonormalization produces vectors u0, u1, and u2 as follows,
 	//
	//   u0 = v0/|v0|
	//   u1 = (v1-(u0*v1)u0)/|v1-(u0*v1)u0|
	//   u2 = (v2-(u0*v2)u0-(u1*v2)u1)/|v2-(u0*v2)u0-(u1*v2)u1|
	//
	// where |A| indicates length of vector A and A*B indicates dot
	// product of vectors A and B.
 	// Gram-Schmidt orthonormalization take linearly independent vectors
 	// U, V, and W and compute an orthonormal set (unit length, mutually
	// perpendicular).

	// compute u0
	u.Normalize();

	// compute u1
	double d0(u%v); 
	v -= u*d0;
	v.Normalize();

	// compute u2
	double d1(v%w);
	d0 = u%w;
	w -= u*d0 + v*d1;
	w.Normalize();
}

std::ostream & operator << (std::ostream &o, const vR3 &v)
{
	o << "v[" << v.x << "," << v.y << "," << v.z << "]" << std::endl;
	return o;
}

//
// m33 interface
//

m33::m33(double  a,double  b,double  c,
         double  d,double  e,double  f,
         double  g,double  h,double  i,
         bool _column)
{
	if (_column)
	{
		vx.x = a; vy.x = d; vz.x = g;
		vx.y = b; vy.y = e; vz.y = h;
		vx.z = c; vy.z = f; vz.z = i;
	}
	else
	{
		vx.x = a; vy.x = b; vz.x = c;
		vx.y = d; vy.y = e; vz.y = f;
		vx.z = g; vy.z = h; vz.z = i;
	}
}

m33::m33(const vR3 &_x,const vR3 &_y,const vR3 &_z):vx(_x),vy(_y),vz(_z) {}

m33 m33::operator + (const m33 &m) const
{
	return m33(vx+m.vx,vy+m.vy,vz+m.vz);
}

m33 & m33::operator += (const m33 &m)
{
	vx+=m.vx; vy+=m.vy; vz+=m.vz;
	return *this;
}

m33 m33::operator - (const m33 &m) const
{
	return m33(vx-m.vx,vy-m.vy,vz-m.vz);
}

m33 & m33::operator -= (const m33 &m)
{
	vx-=m.vx; vy-=m.vy; vz-=m.vz;
	return *this;
}

m33 m33::operator * (double k) const
{
	return m33(vx*k,vy*k,vz*k);
}

m33 & m33::operator *= (double k)
{
	vx*=k; vy*=k; vz*=k;
	return *this;
}

pR3 m33::operator * (const pR3 &p) const
{
	return pR3(p.x*vx.x + p.y*vy.x + p.z*vz.x,
	           p.x*vx.y + p.y*vy.y + p.z*vz.y,
	           p.x*vx.z + p.y*vy.z + p.z*vz.z);
}

vR3 m33::operator * (const vR3 &u) const
{
	return vR3(u.x*vx.x + u.y*vy.x + u.z*vz.x,
	           u.x*vx.y + u.y*vy.y + u.z*vz.y,
	           u.x*vx.z + u.y*vy.z + u.z*vz.z);
}

m33 m33::operator * (const m33 &m) const
{
	return m33(vx.x*m.vx.x + vy.x*m.vx.y + vz.x*m.vx.z,
	           vx.y*m.vx.x + vy.y*m.vx.y + vz.y*m.vx.z,
	           vx.z*m.vx.x + vy.z*m.vx.y + vz.z*m.vx.z,
	           vx.x*m.vy.x + vy.x*m.vy.y + vz.x*m.vy.z,
	           vx.y*m.vy.x + vy.y*m.vy.y + vz.y*m.vy.z,
	           vx.z*m.vy.x + vy.z*m.vy.y + vz.z*m.vy.z,
	           vx.x*m.vz.x + vy.x*m.vz.y + vz.x*m.vz.z,
	           vx.y*m.vz.x + vy.y*m.vz.y + vz.y*m.vz.z,
	           vx.z*m.vz.x + vy.z*m.vz.y + vz.z*m.vz.z);
}

m33 & m33::operator *= (const m33 &m)
{
	*this=*this*m;
	return *this;
}

double m33::operator ! ( void ) const
{
	return (vx.x*vx.x + vx.y*vx.y + vx.z*vx.z +
           vy.x*vy.x + vy.y*vy.y + vy.z*vy.z +
           vz.x*vz.x + vz.y*vz.y + vz.z*vz.z);
}

double m33::Det() const
{
	double det, pos, neg, temp;

	pos = neg = 0.0f;
	temp =  vx.x * vy.y * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.y * vy.z * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.z * vy.x * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.z * vy.y * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.y * vy.x * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.x * vy.z * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	det = pos + neg;

	return det;
}

m33 m33::Inv() const
{
	double det, pos, neg, temp;
	m33 inv;

	// Calculate the determinant of submatrix A
	pos = neg = 0.0f;
	temp =  vx.x * vy.y * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.y * vy.z * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.z * vy.x * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.z * vy.y * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.y * vy.x * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.x * vy.z * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	det = pos + neg;

	// Is Singular ?
	assert((det == 0.0f) || (fabs(det / (pos - neg)) < 1e-15));

	// Calculate inverse(A) = adj(A) / det(A)
	det = 1.0f / det;
	inv.vx.x =( (vy.y * vz.z - vy.z * vz.y) * det);
	inv.vy.x =(-(vy.x * vz.z - vy.z * vz.x) * det);
	inv.vz.x =( (vy.x * vz.y - vy.y * vz.x) * det);
	inv.vx.y =(-(vx.y * vz.z - vx.z * vz.y) * det);
	inv.vy.y =( (vx.x * vz.z - vx.z * vz.x) * det);
	inv.vz.y =(-(vx.x * vz.y - vx.y * vz.x) * det);
	inv.vx.z =( (vx.y * vy.z - vx.z * vy.y) * det);
	inv.vy.z =(-(vx.x * vy.z - vx.z * vy.x) * det);
	inv.vz.z =( (vx.x * vy.y - vx.y * vy.x) * det);

	return inv;
}

m33 m33::Trs() const
{
	return m33(vx.x,vx.y,vx.z,
	           vy.x,vy.y,vy.z,
	           vz.x,vz.y,vz.z,false);
}

ostream & operator << (ostream &o, const m33 &m)
{
	o << "m[" << m.vx.x << " " << m.vy.x << " " << m.vz.x << "]" << endl
	  << " [" << m.vx.y << " " << m.vy.y << " " << m.vz.y << "]" << endl
	  << " [" << m.vx.z << " " << m.vy.z << " " << m.vz.z << "]" << endl;
	return o;
}


//
// Methodes de la classe Transform
//

Transform::Transform(double  a,double  b,double  c,
                   double  d,double  e,double  f,
						 double  g,double  h,double  i,
						 double t1,double t2,double t3) : vx(a,b,c),vy(d,e,f),vz(g,h,i),vt(t1,t2,t3)
{}


Transform::Transform(double x1, double x2, double x3, double x4, 
		  double y1, double y2, double y3, double y4,
		  double z1, double z2, double z3, double z4,
		  double h1, double h2, double h3, double h4) : px(x1, x2, x3, x4), py(y1, y2, y3, y4), pz(z1, z2, z3, z4), ph(h1, h2, h3, h4)
{}

Transform::Transform(const vR3 &X,const vR3 &Y,const vR3 &Z,const vR3 &T):vx(X),vy(Y),vz(Z),vt(T)
{}

Transform Transform::operator + (const Transform &m) const
{
	return Transform(vx+m.vx,vy+m.vy,vz+m.vz,vt+m.vt);
}

Transform & Transform::operator += (const Transform &m)
{
	vx+=m.vx;
	vy+=m.vy;
	vz+=m.vz;
	vt+=m.vt;
	return *this;
}

Transform Transform::operator - (const Transform &m) const
{
	return Transform(vx-m.vx,vy-m.vy,vz-m.vz,vt-m.vt);
}

Transform & Transform::operator -= (const Transform &m)
{
	vx-=m.vx;
	vy-=m.vy;
	vz-=m.vz;
	vt-=m.vt;
	return *this;
}

Transform Transform::operator * (const double r) const
{
	return Transform(vx*r,vy*r,vz*r,vt*r);
}

Transform & Transform::operator *= (const double r)
{
	vx*=r;
	vy*=r;
	vz*=r;
	vt*=r;
	return *this;
}

vR3 Transform::operator * (const vR3 &u) const
{
	return vR3(u.x*vx.x + u.y*vy.x + u.z*vz.x,
	           u.x*vx.y + u.y*vy.y + u.z*vz.y,
				  u.x*vx.z + u.y*vy.z + u.z*vz.z);
}

pR3 Transform::operator * (const pR3 &p) const
{
	return pR3(p.x*vx.x + p.y*vy.x + p.z*vz.x + vt.x,
	           p.x*vx.y + p.y*vy.y + p.z*vz.y + vt.y,
				  p.x*vx.z + p.y*vy.z + p.z*vz.z + vt.z);
}

pR4 Transform::operator *(const pR4& p) const
{
	return pR4(p.coords.x*px.coords.x + p.coords.y*py.coords.x + p.coords.z*pz.coords.x + p.homogen * ph.coords.x,
	           p.coords.x*px.coords.y + p.coords.y*py.coords.y + p.coords.z*pz.coords.y + p.homogen * ph.coords.y,
				  p.coords.x*px.coords.z + p.coords.y*py.coords.z + p.coords.z*pz.coords.z + p.homogen * ph.coords.z);
}		
	
Transform Transform::operator * (const Transform &m) const
{
	return Transform(vx.x*m.vx.x + vy.x*m.vx.y + vz.x*m.vx.z,
	                vx.y*m.vx.x + vy.y*m.vx.y + vz.y*m.vx.z,
						 vx.z*m.vx.x + vy.z*m.vx.y + vz.z*m.vx.z,
						 vx.x*m.vy.x + vy.x*m.vy.y + vz.x*m.vy.z,
						 vx.y*m.vy.x + vy.y*m.vy.y + vz.y*m.vy.z,
						 vx.z*m.vy.x + vy.z*m.vy.y + vz.z*m.vy.z,
						 vx.x*m.vz.x + vy.x*m.vz.y + vz.x*m.vz.z,
						 vx.y*m.vz.x + vy.y*m.vz.y + vz.y*m.vz.z,
						 vx.z*m.vz.x + vy.z*m.vz.y + vz.z*m.vz.z,
						 vx.x*m.vt.x + vy.x*m.vt.y + vz.x*m.vt.z + vt.x,
						 vx.y*m.vt.x + vy.y*m.vt.y + vz.y*m.vt.z + vt.y,
						 vx.z*m.vt.x + vy.z*m.vt.y + vz.z*m.vt.z + vt.z);
}


Transform & Transform::operator *= (const Transform &m)
{
	*this=(*this)*m;
	return *this;
}

Transform Transform::operator / (const double r) const
{
	return Transform(vx/r,vy/r,vz/r,vt/r);
}

Transform & Transform::operator /= (const double r)
{
	vx/=r;
	vy/=r;
	vz/=r;
	vt/=r;
	return *this;
}

double Transform::det() const
{
	double det_1, pos, neg, temp;

	pos = neg = 0.0f;
	temp =  vx.x * vy.y * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.y * vy.z * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.z * vy.x * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.z * vy.y * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.y * vy.x * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.x * vy.z * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	det_1 = pos + neg;

	return det_1;
}

Transform Transform::inv() const
{
	double det_1, pos, neg, temp;
	Transform Inv;

	//  * Calculate the determinant of submatrix A
	pos = neg = 0.0f;
	temp =  vx.x * vy.y * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.y * vy.z * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp =  vx.z * vy.x * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.z * vy.y * vz.x; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.y * vy.x * vz.z; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	temp = -vx.x * vy.z * vz.y; if (temp >= 0.0f) pos += temp;  else  neg += temp;
	det_1 = pos + neg;

	// Is Singular ?
	if ((det_1 == 0.0f) || (fabs(det_1 / (pos - neg)) < 1e-15))
		throw "Transform Transform::inv() const - Singular matrix";

	//  Calculate inverse(A) = adj(A) / det(A)
	det_1 = 1.0f / det_1;
	Inv.vx.x =( (vy.y * vz.z - vy.z * vz.y) * det_1);
	Inv.vy.x =(-(vy.x * vz.z - vy.z * vz.x) * det_1);
	Inv.vz.x =( (vy.x * vz.y - vy.y * vz.x) * det_1);
	Inv.vx.y =(-(vx.y * vz.z - vx.z * vz.y) * det_1);
	Inv.vy.y =( (vx.x * vz.z - vx.z * vz.x) * det_1);
	Inv.vz.y =(-(vx.x * vz.y - vx.y * vz.x) * det_1);
	Inv.vx.z =( (vx.y * vy.z - vx.z * vy.y) * det_1);
	Inv.vy.z =(-(vx.x * vy.z - vx.z * vy.x) * det_1);
	Inv.vz.z =( (vx.x * vy.y - vx.y * vy.x) * det_1);

	Inv.vt.x=-(Inv.vx.x*vt.x+Inv.vy.x*vt.y+Inv.vz.x*vt.z);
	Inv.vt.y=-(Inv.vx.y*vt.x+Inv.vy.y*vt.y+Inv.vz.y*vt.z);
	Inv.vt.z=-(Inv.vx.z*vt.x+Inv.vy.z*vt.y+Inv.vz.z*vt.z);

	return Inv;
}

ostream & operator << (ostream &o, const Transform &m)
{
	o << "m[" << m.vx.x << " " << m.vy.x << " " << m.vz.x << " " << m.vt.x << "]" << endl
	  << " [" << m.vx.y << " " << m.vy.y << " " << m.vz.y << " " << m.vt.y << "]" << endl
	  << " [" << m.vx.z << " " << m.vy.z << " " << m.vz.z << " " << m.vt.z << "]" << endl;
	return o;
}

//
// Methodes de la classe CRep
//

CRep::CRep(){}

CRep::CRep(const vR3 &v1,const vR3 &v2,const vR3 &v3,const vR3 &v4):mat(v1,v2,v3,v4)
{
	invMat=mat.inv();
}

CRep::CRep(const Transform &m):mat(m),invMat(m.inv()){}

CRep CRep::inv() const
{
	CRep reverse;
	reverse.mat=invMat;
	reverse.invMat=mat;

	return reverse;
}

CRep CRep::operator * (const CRep &cr) const
{
	return CRep(mat*cr.mat);
}

CRep & CRep::operator *= (const CRep &cr)
{
	mat*=cr.mat;
	invMat=cr.invMat*invMat;

	return *this;
}

vR3 CRep::Apply(const vR3 &v) const
{
	return mat*v;
}

vR3 CRep::invApply(const vR3 &v) const
{
	return invMat*v;
}

pR3 CRep::Apply(const pR3 &p) const
{
	return mat*p;
}

pR3 CRep::invApply(const pR3 &p) const
{
	return invMat*p;
}
/*
std::ostream & operator << (std::ostream &o,const CRep &cr)
{
	o << "CRep\n" << cr.mat << endl << "    \n" << cr.invMat;
	return o;
}
*/
//
// Methodes de la classe Trans
//

Trans::Trans(double a,double b,double c) : CRep(Transform(1,0,0,0,1,0,0,0,1,a,b,c)){}

Trans::Trans(const vR3 &v):CRep(Transform(1,0,0,0,1,0,0,0,1,v.x,v.y,v.z)){}

CRep Trans::operator * (const CRep &cr) const
{
	Transform tmp(cr.mat);
	tmp.vt+=mat.vt;

	return CRep(tmp);
}

CRep & Trans::operator *= (const CRep &cr)
{
	Transform tmp(cr.mat);
	tmp.vt+=mat.vt;
	mat=tmp;
	invMat=tmp.inv();

	return *this;
}

vR3 Trans::Apply(const vR3 &v) const
{
	return v;
}

vR3 Trans::invApply(const vR3 &v) const
{
	return v;
}

pR3 Trans::Apply(const pR3 &p) const
{
	return pR3(p+mat.vt);
}

pR3 Trans::invApply(const pR3 &p) const
{
	return pR3(p-mat.vt);
}

//
// Methodes de la classe RotX
//

RotX::RotX(double theta):CRep(Transform(1.,0.,0.,0.,cos(theta),sin(theta),0.,-sin(theta),cos(theta))){}

//
// Methodes de la classe RotY
//

RotY::RotY(double theta):CRep(Transform(cos(theta),0.,-sin(theta),0.,1.,0.,sin(theta),0.,cos(theta))){}

//
// Methodes de la classe RotZ
//

RotZ::RotZ(double theta):CRep(Transform(cos(theta),sin(theta),0.,-sin(theta),cos(theta),0.,0.,0.,1.)){}

//
// Methodes de la classe RotA
//

RotA::RotA(double x,double y,double z,double theta):CRep(Transform(x*x+(1-x*x)*cos(theta),x*y*(1-cos(theta))+z*sin(theta),x*z*(1-cos(theta))-y*sin(theta),
                                                                  x*y*(1-cos(theta))-z*sin(theta),y*y+cos(theta)*(1-y*y),y*z*(1-cos(theta))+x*sin(theta),
																						x*z*(1-cos(theta))+y*sin(theta),y*y*(1-cos(theta))+x*sin(theta),x*x+cos(theta)*(1-z*z))){}

RotA::RotA(const vR3 &v,double theta):CRep(Transform(v.x*v.x+(1-v.x*v.x)*cos(theta),v.x*v.y*(1-cos(theta))+v.z*sin(theta),v.x*v.z*(1-cos(theta))-v.y*sin(theta),
                                                    v.x*v.y*(1-cos(theta))-v.z*sin(theta),v.y*v.y+cos(theta)*(1-v.y*v.y),v.y*v.z*(1-cos(theta))+v.x*sin(theta),
																	 v.x*v.z*(1-cos(theta))+v.y*sin(theta),v.y*v.y*(1-cos(theta))+v.x*sin(theta),v.x*v.x+cos(theta)*(1-v.z*v.z))){}

//
// Methodes de la classe Scale
//

Scale::Scale(double x,double y,double z):CRep(Transform(x,0,0,0,y,0,0,0,z)){}

pR4::pR4(const pR3& p) : coords(p), homogen(1.)
{}

pR4::pR4(const double& _x, const double& _y, const double& _z) : coords(_x, _y, _z), homogen(1.)
{}

pR4::pR4(const double& _x, const double& _y, const double& _z, const double& _h) : coords(_x, _y, _z), homogen(_h)
{}

pR4& pR4::operator =(const pR3& src)
{
	coords  = src;
	homogen = 1.;
	
	return *this;
}

pR4& pR4::operator =(const pR4& src)
{
	coords  = src.coords;
	homogen = src.homogen;
	 
	return *this;
}
