// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */

#include <motor/collision.hh>

bool Box2BoxCollision(SBox* obj_tmp, SBox* _box)
{	
	const vR3* akA = obj_tmp->getAllAxis();
	const vR3* akB = _box->getAllAxis(); 

	 double afEA[3] = { obj_tmp->getLong() / 2., obj_tmp->getHeight() / 2., obj_tmp->getWidth() / 2. };
	 double afEB[3] = { _box->getLong() / 2., _box->getHeight() / 2., _box->getWidth() / 2. };
 
    	// compute difference of box centers, D = C1-C0
    	vR3 kD = vR3(obj_tmp->getMassCenter(), _box->getMassCenter());

    	double aafC[3][3];    
    	double aafAbsC[3][3];  
    	double afAD[3];        
    	double fR0, fR1, fR;  
    	double fR01;      
    		    
	aafC[0][0] = akA[0].Dot(akB[0]);
    	aafC[0][1] = akA[0].Dot(akB[1]);
    	aafC[0][2] = akA[0].Dot(akB[2]);
    	afAD[0] = akA[0].Dot(kD);
    	aafAbsC[0][0] = fabs(aafC[0][0]);
    	aafAbsC[0][1] = fabs(aafC[0][1]);
    	aafAbsC[0][2] = fabs(aafC[0][2]);
    	fR = fabs(afAD[0]);
    	fR1 = afEB[0]*aafAbsC[0][0]+afEB[1]*aafAbsC[0][1]+afEB[2]*aafAbsC[0][2];
    	fR01 = afEA[0] + fR1;
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A1
    	aafC[1][0] = akA[1].Dot(akB[0]);
    	aafC[1][1] = akA[1].Dot(akB[1]);
    	aafC[1][2] = akA[1].Dot(akB[2]);
    	afAD[1] = akA[1].Dot(kD);
    	aafAbsC[1][0] = fabs(aafC[1][0]);
    	aafAbsC[1][1] = fabs(aafC[1][1]);
    	aafAbsC[1][2] = fabs(aafC[1][2]);
    	fR = fabs(afAD[1]);
    	fR1 = afEB[0]*aafAbsC[1][0]+afEB[1]*aafAbsC[1][1]+afEB[2]*aafAbsC[1][2];
    	fR01 = afEA[1] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*A2
    	aafC[2][0] = akA[2].Dot(akB[0]);
    	aafC[2][1] = akA[2].Dot(akB[1]);
    	aafC[2][2] = akA[2].Dot(akB[2]);
    	afAD[2] = akA[2].Dot(kD);
    	aafAbsC[2][0] = fabs(aafC[2][0]);
    	aafAbsC[2][1] = fabs(aafC[2][1]);
    	aafAbsC[2][2] = fabs(aafC[2][2]);
    	fR = fabs(afAD[2]);
    	fR1 = afEB[0]*aafAbsC[2][0]+afEB[1]*aafAbsC[2][1]+afEB[2]*aafAbsC[2][2];
    	fR01 = afEA[2] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*B0
    	fR = fabs(akB[0].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][0]+afEA[1]*aafAbsC[1][0]+afEA[2]*aafAbsC[2][0];
    	fR01 = fR0 + afEB[0];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B1
    	fR = fabs(akB[1].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][1]+afEA[1]*aafAbsC[1][1]+afEA[2]*aafAbsC[2][1];
    	fR01 = fR0 + afEB[1];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B2
    	fR = fabs(akB[2].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][2]+afEA[1]*aafAbsC[1][2]+afEA[2]*aafAbsC[2][2];
    	fR01 = fR0 + afEB[2];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A0xB0
    	fR = fabs(afAD[2]*aafC[1][0]-afAD[1]*aafC[2][0]);
    	fR0 = afEA[1]*aafAbsC[2][0] + afEA[2]*aafAbsC[1][0];
    	fR1 = afEB[1]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][1];
    	fR01 = fR0 + fR1;
    	if ( fR > fR01 )
        	return false;

	// axis C0+t*A0xB1
	fR = fabs(afAD[2]*aafC[1][1]-afAD[1]*aafC[2][1]);
	fR0 = afEA[1]*aafAbsC[2][1] + afEA[2]*aafAbsC[1][1];
	fR1 = afEB[0]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A0xB2
	fR = fabs(afAD[2]*aafC[1][2]-afAD[1]*aafC[2][2]);
	fR0 = afEA[1]*aafAbsC[2][2] + afEA[2]*aafAbsC[1][2];
	fR1 = afEB[0]*aafAbsC[0][1] + afEB[1]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB0
	fR = fabs(afAD[0]*aafC[2][0]-afAD[2]*aafC[0][0]);
	fR0 = afEA[0]*aafAbsC[2][0] + afEA[2]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB1
	fR = fabs(afAD[0]*aafC[2][1]-afAD[2]*aafC[0][1]);
	fR0 = afEA[0]*aafAbsC[2][1] + afEA[2]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB2
	fR = fabs(afAD[0]*aafC[2][2]-afAD[2]*aafC[0][2]);
	fR0 = afEA[0]*aafAbsC[2][2] + afEA[2]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[1][1] + afEB[1]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB0
	fR = fabs(afAD[1]*aafC[0][0]-afAD[0]*aafC[1][0]);
	fR0 = afEA[0]*aafAbsC[1][0] + afEA[1]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB1
	fR = fabs(afAD[1]*aafC[0][1]-afAD[0]*aafC[1][1]);
	fR0 = afEA[0]*aafAbsC[1][1] + afEA[1]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB2
	fR = fabs(afAD[1]*aafC[0][2]-afAD[0]*aafC[1][2]);
	fR0 = afEA[0]*aafAbsC[1][2] + afEA[1]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[2][1] + afEB[1]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	return true;
}

/* -------------------- Class Collision Plan / Sphere detection Methods -------------------- */		

bool SPlanSphere::collisionDetected(const SInteraction& inter,const SPlan& _plane) const
{
	vR3 sNorm(-_plane.getNormal());
	double R(((SSphere*)(*(inter.sObj)))->getRadius());
	double sConst(_plane.getConstant());
	double length(sqrt(!sNorm));
	
	sNorm *= 1./length;
	sConst /= length;
	
	double sDist(sNorm % vR3(inter.sPt) - sConst);
	return ( ((double)fabs(sDist)) <= R );
}

bool SPlanSphere::collisionDetected(const SInteraction& inter,const SLine& _line) const
{
	pR3 R(inter.sPt);
	vR3 sDiff(R - _line.getOrigin());
	double sqlen = !(_line.getDirection());
	double _T = (sDiff % _line.getDirection()) / sqlen;
	double sDist = !sDiff;
	return sDist <= ((SSphere*)(*(inter.sObj)))->getRadius();
}

bool SPlanSphere::collisionDetected(const SInteraction& inter,const SBox& _box) const
{
	vR3 kDiff(inter.sPt - _box.getMassCenter());
	double fLen = !kDiff;
	
	double tmp[3] = 
	{
		(kDiff.Normalize() % _box.getAxis(0)) * _box.getLong() / 2.,
		(kDiff.Normalize() % _box.getAxis(1)) * _box.getHeight() / 2.,
		(kDiff.Normalize() % _box.getAxis(2)) * _box.getWidth() / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]) + ((SSphere*)(*(inter.sObj)))->getRadius();		
		
	return mLength >= fLen;
}

bool SPlanSphere::collisionDetected(const SInteraction& inter,const SSphere& _sph) const
{
		
	vR3 kDiff(inter.sPt - _sph.getMassCenter());
	double fLen(!kDiff);
	double fSum(((SSphere*)(*(inter.sObj)))->getRadius() + _sph.getRadius());
	double fSumSq(SQ(fSum));
	
	
	return ( fLen <= fSumSq );
}


bool SPlanSphere::collisionDetected(const SInteraction& inter, const SPerso& perso) const
{
	SBox* _box = perso.boundingBox();
	vR3 kDiff(inter.sPt - _box->getMassCenter());
	double fLen = !kDiff;
	
	double tmp[3] = 
	{
		(kDiff.Normalize() % _box->getAxis(0)) * _box->getLong() / 2.,
		(kDiff.Normalize() % _box->getAxis(1)) * _box->getHeight() / 2.,
		(kDiff.Normalize() % _box->getAxis(2)) * _box->getWidth() / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]) + ((SSphere*)(*(inter.sObj)))->getRadius();		
		
	return mLength >= fLen;
	return false;
}



/* -------------------- Class Collision Box / Plan / Sphere detection Methods -------------------- */	


bool SPlanBox::collisionDetected(const SInteraction& inter,const SPlan& _plane) const
{
	double tmp[3] = 
	{
		(_plane.getNormal() % ((SBox*)(*(inter.sObj)))->getAxis(0)) * ((SBox*)(*(inter.sObj)))->getLong() / 2.,
		(_plane.getNormal() % ((SBox*)(*(inter.sObj)))->getAxis(1)) * ((SBox*)(*(inter.sObj)))->getHeight() / 2.,
		(_plane.getNormal() % ((SBox*)(*(inter.sObj)))->getAxis(2)) * ((SBox*)(*(inter.sObj)))->getWidth() / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]);
	double psdist = _plane.getDistanceTo(pR3(inter.sPt.x, inter.sPt.y /*- ((SBox*)(*(inter.sObj)))->getHeight() / 2.*/, inter.sPt.z));
	
	return ((double)fabs(psdist)) <= mLength;
}

bool SPlanBox::collisionDetected(const SInteraction& inter,const SLine& _line) const
{
	return false;
}

bool SPlanBox::collisionDetected(const SInteraction& inter,const SSphere& _sph) const
{	
	SBox* _box = (SBox*)*(inter.sObj);
	vR3 kDiff(inter.sPt - _sph.getMassCenter());
	double fLen = !kDiff;
	
	double tmp[3] = 
	{
		(kDiff.Normalize() % _box->getAxis(0)) * ((SBox*)(*(inter.sObj)))->getLong() / 2.,
		(kDiff.Normalize() % _box->getAxis(1)) * ((SBox*)(*(inter.sObj)))->getHeight() / 2.,
		(kDiff.Normalize() % _box->getAxis(2)) * ((SBox*)(*(inter.sObj)))->getWidth() / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]) + _sph.getRadius();		
		
	return mLength >= fLen;
}

bool SPlanBox::collisionDetected(const SInteraction& inter, const SBox& _box) const
{ 
	
	SBox* obj_tmp = (SBox*)*(inter.sObj);
	
	const vR3* akA = ((SBox*)(*(inter.sObj)))->getAllAxis();
	const vR3* akB = _box.getAllAxis(); 

	 double afEA[3] = { obj_tmp->getLong() / 2., obj_tmp->getHeight() / 2., obj_tmp->getWidth() / 2. };
	 double afEB[3] = { _box.getLong() / 2., _box.getHeight() / 2., _box.getWidth() / 2. };
 
    	// compute difference of box centers, D = C1-C0
    	vR3 kD = vR3(inter.sPt, _box.getMassCenter());

    	double aafC[3][3];    
    	double aafAbsC[3][3];  
    	double afAD[3];        
    	double fR0, fR1, fR;  
    	double fR01;      
    		    
	aafC[0][0] = akA[0].Dot(akB[0]);
    	aafC[0][1] = akA[0].Dot(akB[1]);
    	aafC[0][2] = akA[0].Dot(akB[2]);
    	afAD[0] = akA[0].Dot(kD);
    	aafAbsC[0][0] = fabs(aafC[0][0]);
    	aafAbsC[0][1] = fabs(aafC[0][1]);
    	aafAbsC[0][2] = fabs(aafC[0][2]);
    	fR = fabs(afAD[0]);
    	fR1 = afEB[0]*aafAbsC[0][0]+afEB[1]*aafAbsC[0][1]+afEB[2]*aafAbsC[0][2];
    	fR01 = afEA[0] + fR1;
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A1
    	aafC[1][0] = akA[1].Dot(akB[0]);
    	aafC[1][1] = akA[1].Dot(akB[1]);
    	aafC[1][2] = akA[1].Dot(akB[2]);
    	afAD[1] = akA[1].Dot(kD);
    	aafAbsC[1][0] = fabs(aafC[1][0]);
    	aafAbsC[1][1] = fabs(aafC[1][1]);
    	aafAbsC[1][2] = fabs(aafC[1][2]);
    	fR = fabs(afAD[1]);
    	fR1 = afEB[0]*aafAbsC[1][0]+afEB[1]*aafAbsC[1][1]+afEB[2]*aafAbsC[1][2];
    	fR01 = afEA[1] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*A2
    	aafC[2][0] = akA[2].Dot(akB[0]);
    	aafC[2][1] = akA[2].Dot(akB[1]);
    	aafC[2][2] = akA[2].Dot(akB[2]);
    	afAD[2] = akA[2].Dot(kD);
    	aafAbsC[2][0] = fabs(aafC[2][0]);
    	aafAbsC[2][1] = fabs(aafC[2][1]);
    	aafAbsC[2][2] = fabs(aafC[2][2]);
    	fR = fabs(afAD[2]);
    	fR1 = afEB[0]*aafAbsC[2][0]+afEB[1]*aafAbsC[2][1]+afEB[2]*aafAbsC[2][2];
    	fR01 = afEA[2] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*B0
    	fR = fabs(akB[0].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][0]+afEA[1]*aafAbsC[1][0]+afEA[2]*aafAbsC[2][0];
    	fR01 = fR0 + afEB[0];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B1
    	fR = fabs(akB[1].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][1]+afEA[1]*aafAbsC[1][1]+afEA[2]*aafAbsC[2][1];
    	fR01 = fR0 + afEB[1];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B2
    	fR = fabs(akB[2].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][2]+afEA[1]*aafAbsC[1][2]+afEA[2]*aafAbsC[2][2];
    	fR01 = fR0 + afEB[2];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A0xB0
    	fR = fabs(afAD[2]*aafC[1][0]-afAD[1]*aafC[2][0]);
    	fR0 = afEA[1]*aafAbsC[2][0] + afEA[2]*aafAbsC[1][0];
    	fR1 = afEB[1]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][1];
    	fR01 = fR0 + fR1;
    	if ( fR > fR01 )
        	return false;

	// axis C0+t*A0xB1
	fR = fabs(afAD[2]*aafC[1][1]-afAD[1]*aafC[2][1]);
	fR0 = afEA[1]*aafAbsC[2][1] + afEA[2]*aafAbsC[1][1];
	fR1 = afEB[0]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A0xB2
	fR = fabs(afAD[2]*aafC[1][2]-afAD[1]*aafC[2][2]);
	fR0 = afEA[1]*aafAbsC[2][2] + afEA[2]*aafAbsC[1][2];
	fR1 = afEB[0]*aafAbsC[0][1] + afEB[1]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB0
	fR = fabs(afAD[0]*aafC[2][0]-afAD[2]*aafC[0][0]);
	fR0 = afEA[0]*aafAbsC[2][0] + afEA[2]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB1
	fR = fabs(afAD[0]*aafC[2][1]-afAD[2]*aafC[0][1]);
	fR0 = afEA[0]*aafAbsC[2][1] + afEA[2]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB2
	fR = fabs(afAD[0]*aafC[2][2]-afAD[2]*aafC[0][2]);
	fR0 = afEA[0]*aafAbsC[2][2] + afEA[2]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[1][1] + afEB[1]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB0
	fR = fabs(afAD[1]*aafC[0][0]-afAD[0]*aafC[1][0]);
	fR0 = afEA[0]*aafAbsC[1][0] + afEA[1]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB1
	fR = fabs(afAD[1]*aafC[0][1]-afAD[0]*aafC[1][1]);
	fR0 = afEA[0]*aafAbsC[1][1] + afEA[1]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB2
	fR = fabs(afAD[1]*aafC[0][2]-afAD[0]*aafC[1][2]);
	fR0 = afEA[0]*aafAbsC[1][2] + afEA[1]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[2][1] + afEB[1]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	return true;
}

bool SPlanBox::collisionDetected(const SInteraction& inter, const SPerso& perso) const
{
	SBox* obj_tmp = perso.boundingBox();
	SBox* _box    = ((SBox*)*(inter.sObj));
	
	const vR3* akA = obj_tmp->getAllAxis();
    	const vR3* akB = _box->getAllAxis();
	
	double afEA[3] = { obj_tmp->getLong() / 2., obj_tmp->getHeight() / 2., obj_tmp->getWidth() / 2. };
	double afEB[3] = { _box->getLong() / 2., _box->getHeight() / 2., _box->getWidth() / 2. };
	
	vR3 kD = vR3(inter.sPt, obj_tmp->getMassCenter());

	double aafC[3][3];     
	double aafAbsC[3][3]; 
	double afAD[3];        
	double fR0, fR1, fR;   
	double fR01;          

	aafC[0][0] = akA[0].Dot(akB[0]);
    	aafC[0][1] = akA[0].Dot(akB[1]);
    	aafC[0][2] = akA[0].Dot(akB[2]);
    	afAD[0] = akA[0].Dot(kD);
    	aafAbsC[0][0] = fabs(aafC[0][0]);
    	aafAbsC[0][1] = fabs(aafC[0][1]);
    	aafAbsC[0][2] = fabs(aafC[0][2]);
    	fR = fabs(afAD[0]);
    	fR1 = afEB[0]*aafAbsC[0][0]+afEB[1]*aafAbsC[0][1]+afEB[2]*aafAbsC[0][2];
    	fR01 = afEA[0] + fR1;
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A1
    	aafC[1][0] = akA[1].Dot(akB[0]);
    	aafC[1][1] = akA[1].Dot(akB[1]);
    	aafC[1][2] = akA[1].Dot(akB[2]);
    	afAD[1] = akA[1].Dot(kD);
    	aafAbsC[1][0] = fabs(aafC[1][0]);
    	aafAbsC[1][1] = fabs(aafC[1][1]);
    	aafAbsC[1][2] = fabs(aafC[1][2]);
    	fR = fabs(afAD[1]);
    	fR1 = afEB[0]*aafAbsC[1][0]+afEB[1]*aafAbsC[1][1]+afEB[2]*aafAbsC[1][2];
    	fR01 = afEA[1] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*A2
    	aafC[2][0] = akA[2].Dot(akB[0]);
    	aafC[2][1] = akA[2].Dot(akB[1]);
    	aafC[2][2] = akA[2].Dot(akB[2]);
    	afAD[2] = akA[2].Dot(kD);
    	aafAbsC[2][0] = fabs(aafC[2][0]);
    	aafAbsC[2][1] = fabs(aafC[2][1]);
    	aafAbsC[2][2] = fabs(aafC[2][2]);
    	fR = fabs(afAD[2]);
    	fR1 = afEB[0]*aafAbsC[2][0]+afEB[1]*aafAbsC[2][1]+afEB[2]*aafAbsC[2][2];
    	fR01 = afEA[2] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*B0
    	fR = fabs(akB[0].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][0]+afEA[1]*aafAbsC[1][0]+afEA[2]*aafAbsC[2][0];
    	fR01 = fR0 + afEB[0];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B1
    	fR = fabs(akB[1].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][1]+afEA[1]*aafAbsC[1][1]+afEA[2]*aafAbsC[2][1];
    	fR01 = fR0 + afEB[1];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B2
    	fR = fabs(akB[2].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][2]+afEA[1]*aafAbsC[1][2]+afEA[2]*aafAbsC[2][2];
    	fR01 = fR0 + afEB[2];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A0xB0
    	fR = fabs(afAD[2]*aafC[1][0]-afAD[1]*aafC[2][0]);
    	fR0 = afEA[1]*aafAbsC[2][0] + afEA[2]*aafAbsC[1][0];
    	fR1 = afEB[1]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][1];
    	fR01 = fR0 + fR1;
    	if ( fR > fR01 )
        	return false;

	// axis C0+t*A0xB1
	fR = fabs(afAD[2]*aafC[1][1]-afAD[1]*aafC[2][1]);
	fR0 = afEA[1]*aafAbsC[2][1] + afEA[2]*aafAbsC[1][1];
	fR1 = afEB[0]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A0xB2
	fR = fabs(afAD[2]*aafC[1][2]-afAD[1]*aafC[2][2]);
	fR0 = afEA[1]*aafAbsC[2][2] + afEA[2]*aafAbsC[1][2];
	fR1 = afEB[0]*aafAbsC[0][1] + afEB[1]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB0
	fR = fabs(afAD[0]*aafC[2][0]-afAD[2]*aafC[0][0]);
	fR0 = afEA[0]*aafAbsC[2][0] + afEA[2]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB1
	fR = fabs(afAD[0]*aafC[2][1]-afAD[2]*aafC[0][1]);
	fR0 = afEA[0]*aafAbsC[2][1] + afEA[2]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB2
	fR = fabs(afAD[0]*aafC[2][2]-afAD[2]*aafC[0][2]);
	fR0 = afEA[0]*aafAbsC[2][2] + afEA[2]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[1][1] + afEB[1]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB0
	fR = fabs(afAD[1]*aafC[0][0]-afAD[0]*aafC[1][0]);
	fR0 = afEA[0]*aafAbsC[1][0] + afEA[1]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB1
	fR = fabs(afAD[1]*aafC[0][1]-afAD[0]*aafC[1][1]);
	fR0 = afEA[0]*aafAbsC[1][1] + afEA[1]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB2
	fR = fabs(afAD[1]*aafC[0][2]-afAD[0]*aafC[1][2]);
	fR0 = afEA[0]*aafAbsC[1][2] + afEA[1]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[2][1] + afEB[1]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	return true;
			
}

/* ---------------- Classe SPesro Collision ----------------------- */

bool SPersoCol::collisionDetected(const SInteraction& inter, const SPlan& _plane) const
{
	double tmp[3] = 
	{
		(_plane.getNormal() % ((SPerso*)(*(inter.sObj)))->boundingBox()->getAxis(0)) * ((SPerso*)(*(inter.sObj)))->boundingBox()->getLong() / 2.,
		(_plane.getNormal() % ((SPerso*)(*(inter.sObj)))->boundingBox()->getAxis(1)) * ((SPerso*)(*(inter.sObj)))->boundingBox()->getHeight() / 2.,
		(_plane.getNormal() % ((SPerso*)(*(inter.sObj)))->boundingBox()->getAxis(2)) * ((SPerso*)(*(inter.sObj)))->boundingBox()->getWidth() / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]);
	double psdist = _plane.getDistanceTo(inter.sPt);
	
	return ((double)fabs(psdist)) <= mLength;
}

bool SPersoCol::collisionDetected(const SInteraction&, const SLine&) const
{
	return false;
}

bool SPersoCol::collisionDetected(const SInteraction& inter, const SBox& _box) const
{
	SBox* obj_tmp = ((SPerso*)*(inter.sObj))->boundingBox();
	pR3 tmpP(_box.getMassCenter());
	
	const vR3* akA = obj_tmp->getAllAxis();
    	const vR3* akB = _box.getAllAxis();
	
	double afEA[3] = { obj_tmp->getLong() / 2., obj_tmp->getHeight() / 2., obj_tmp->getWidth() / 2. };
	double afEB[3] = { _box.getLong() / 2., _box.getHeight() / 2., _box.getWidth() / 2. };

	vR3 kD = vR3(inter.sPt, _box.getMassCenter());

	double aafC[3][3];     
	double aafAbsC[3][3]; 
	double afAD[3];        
	double fR0, fR1, fR;   
	double fR01;          

	aafC[0][0] = akA[0].Dot(akB[0]);
    	aafC[0][1] = akA[0].Dot(akB[1]);
    	aafC[0][2] = akA[0].Dot(akB[2]);
    	afAD[0] = akA[0].Dot(kD);
    	aafAbsC[0][0] = fabs(aafC[0][0]);
    	aafAbsC[0][1] = fabs(aafC[0][1]);
    	aafAbsC[0][2] = fabs(aafC[0][2]);
    	fR = fabs(afAD[0]);
    	fR1 = afEB[0]*aafAbsC[0][0]+afEB[1]*aafAbsC[0][1]+afEB[2]*aafAbsC[0][2];
    	fR01 = afEA[0] + fR1;
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A1
    	aafC[1][0] = akA[1].Dot(akB[0]);
    	aafC[1][1] = akA[1].Dot(akB[1]);
    	aafC[1][2] = akA[1].Dot(akB[2]);
    	afAD[1] = akA[1].Dot(kD);
    	aafAbsC[1][0] = fabs(aafC[1][0]);
    	aafAbsC[1][1] = fabs(aafC[1][1]);
    	aafAbsC[1][2] = fabs(aafC[1][2]);
    	fR = fabs(afAD[1]);
    	fR1 = afEB[0]*aafAbsC[1][0]+afEB[1]*aafAbsC[1][1]+afEB[2]*aafAbsC[1][2];
    	fR01 = afEA[1] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*A2
    	aafC[2][0] = akA[2].Dot(akB[0]);
    	aafC[2][1] = akA[2].Dot(akB[1]);
    	aafC[2][2] = akA[2].Dot(akB[2]);
    	afAD[2] = akA[2].Dot(kD);
    	aafAbsC[2][0] = fabs(aafC[2][0]);
    	aafAbsC[2][1] = fabs(aafC[2][1]);
    	aafAbsC[2][2] = fabs(aafC[2][2]);
    	fR = fabs(afAD[2]);
    	fR1 = afEB[0]*aafAbsC[2][0]+afEB[1]*aafAbsC[2][1]+afEB[2]*aafAbsC[2][2];
    	fR01 = afEA[2] + fR1;
    	if ( fR > fR01 )
       		return false;

    	// axis C0+t*B0
    	fR = fabs(akB[0].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][0]+afEA[1]*aafAbsC[1][0]+afEA[2]*aafAbsC[2][0];
    	fR01 = fR0 + afEB[0];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B1
    	fR = fabs(akB[1].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][1]+afEA[1]*aafAbsC[1][1]+afEA[2]*aafAbsC[2][1];
    	fR01 = fR0 + afEB[1];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*B2
    	fR = fabs(akB[2].Dot(kD));
    	fR0 = afEA[0]*aafAbsC[0][2]+afEA[1]*aafAbsC[1][2]+afEA[2]*aafAbsC[2][2];
    	fR01 = fR0 + afEB[2];
    	if ( fR > fR01 )
        	return false;

    	// axis C0+t*A0xB0
    	fR = fabs(afAD[2]*aafC[1][0]-afAD[1]*aafC[2][0]);
    	fR0 = afEA[1]*aafAbsC[2][0] + afEA[2]*aafAbsC[1][0];
    	fR1 = afEB[1]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][1];
    	fR01 = fR0 + fR1;
    	if ( fR > fR01 )
        	return false;

	// axis C0+t*A0xB1
	fR = fabs(afAD[2]*aafC[1][1]-afAD[1]*aafC[2][1]);
	fR0 = afEA[1]*aafAbsC[2][1] + afEA[2]*aafAbsC[1][1];
	fR1 = afEB[0]*aafAbsC[0][2] + afEB[2]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A0xB2
	fR = fabs(afAD[2]*aafC[1][2]-afAD[1]*aafC[2][2]);
	fR0 = afEA[1]*aafAbsC[2][2] + afEA[2]*aafAbsC[1][2];
	fR1 = afEB[0]*aafAbsC[0][1] + afEB[1]*aafAbsC[0][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB0
	fR = fabs(afAD[0]*aafC[2][0]-afAD[2]*aafC[0][0]);
	fR0 = afEA[0]*aafAbsC[2][0] + afEA[2]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB1
	fR = fabs(afAD[0]*aafC[2][1]-afAD[2]*aafC[0][1]);
	fR0 = afEA[0]*aafAbsC[2][1] + afEA[2]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[1][2] + afEB[2]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A1xB2
	fR = fabs(afAD[0]*aafC[2][2]-afAD[2]*aafC[0][2]);
	fR0 = afEA[0]*aafAbsC[2][2] + afEA[2]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[1][1] + afEB[1]*aafAbsC[1][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB0
	fR = fabs(afAD[1]*aafC[0][0]-afAD[0]*aafC[1][0]);
	fR0 = afEA[0]*aafAbsC[1][0] + afEA[1]*aafAbsC[0][0];
	fR1 = afEB[1]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][1];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB1
	fR = fabs(afAD[1]*aafC[0][1]-afAD[0]*aafC[1][1]);
	fR0 = afEA[0]*aafAbsC[1][1] + afEA[1]*aafAbsC[0][1];
	fR1 = afEB[0]*aafAbsC[2][2] + afEB[2]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	// axis C0+t*A2xB2
	fR = fabs(afAD[1]*aafC[0][2]-afAD[0]*aafC[1][2]);
	fR0 = afEA[0]*aafAbsC[1][2] + afEA[1]*aafAbsC[0][2];
	fR1 = afEB[0]*aafAbsC[2][1] + afEB[1]*aafAbsC[2][0];
	fR01 = fR0 + fR1;
	if ( fR > fR01 )
            return false;

	return true;
}

bool SPersoCol::collisionDetected(const SInteraction& inter, const SSphere& _sph) const
{
	SBox* _box = ((SPerso*)(*(inter.sObj)))->boundingBox();
	vR3 kDiff(inter.sPt - _sph.getMassCenter());
	double fLen = !kDiff;
	
	double tmp[3] = 
	{
		(kDiff.Normalize() % _box->getAxis(0)) * _box->getLong()   / 2.,
		(kDiff.Normalize() % _box->getAxis(1)) * _box->getHeight() / 2.,
		(kDiff.Normalize() % _box->getAxis(2)) * _box->getWidth()  / 2.
	};
	
	double mLength = fabs(tmp[0]) + fabs(tmp[1]) + fabs(tmp[2]) + _sph.getRadius();		
		
	return mLength >= fLen;
}

bool SPersoCol::collisionDetected(const SInteraction&, const SPerso&) const
{
	return false;
}
		
