// color.cc v 1.0 09.2003 (mGC Library)
// Definition of cN3 cR3 classes
// Copyright (C) 2003 L.E.R.I.  All rights reserved.
// Reims University (FRANCE)

#include <iostream>
#include <cassert>


#include <color.h>

using namespace std;
using namespace mCG;

const cN3 cN3::BLACK      (  0,	 0,   0);
const cN3 cN3::WHITE      (255, 255, 255);
const cN3 cN3::DARKGRAY   (128, 128, 128);
const cN3 cN3::GRAY       (160, 160, 164);
const cN3 cN3::LIGHTGRAY  (192, 192, 192);
const cN3 cN3::RED 		  (255,  0,    0);
const cN3 cN3::GREEN		  (  0, 255,	0);
const cN3 cN3::BLUE		  (  0,   0, 255);
const cN3 cN3::CYAN		  (  0, 255, 255);
const cN3 cN3::MAGENTA	  (255,   0, 255);
const cN3 cN3::YELLOW	  (255, 255,	0);
const cN3 cN3::DARKRED	  (128,   0,   0);
const cN3 cN3::DARKGREEN  (  0, 128,	0);
const cN3 cN3::DARKBLUE	  (  0,   0, 128);
const cN3 cN3::DARKCYAN	  (  0, 128, 128);
const cN3 cN3::DARKMAGENTA(128,   0, 128);
const cN3 cN3::DARKYELLOW (128, 128,	0);

//
// cN3 interface
//

cN3::cN3(int _x, int _y, int _z, TypeColor tc)
{
	if (tc == cN3::Hsv) setHsv(_x,_y,_z); else setRgb(_x,_y,_z);
}

cN3::cN3(const cR3 &c)
{
	setRgb(ROUND(c.r),ROUND(c.g),ROUND(c.b));
}

void cN3::rgb(int *r, int *g, int *b) const
{
	*r = (int)( value        & 0xff);
	*g = (int)((value >>  8) & 0xff);
	*b = (int)((value >> 16) & 0xff);
}

void cN3::hsv(int *h, int *s, int *v) const
{
	int r = (int)( value        & 0xff);
	int g = (int)((value >>  8) & 0xff);
	int b = (int)((value >> 16) & 0xff);
	UINT max(r);
	int whatmax(0);
	if ((UINT)g > max) { max = g; whatmax = 1; }
	if ((UINT)b > max) { max = b; whatmax = 2; }
	UINT min(r);
	if ((UINT)g < min) min = g;
	if ((UINT)b < min) min = b;
	int delta(max-min);
	*v = max;
	*s = max ? (510*delta+max)/(2*max) : 0;
	if ( *s == 0 ) {
		*h = -1;
	} else {
		switch ( whatmax ) {
			case 0:
				if ( g >= b ) *h = (120*(g-b)+delta)/(2*delta);
				else          *h = (120*(g-b+delta)+delta)/(2*delta) + 300;
				break;
			case 1:
				if ( b  > r ) *h = 120 + (120*(b-r)+delta)/(2*delta);
				else          *h = 60  + (120*(b-r+delta)+delta)/(2*delta);
				break;
			case 2:
				if ( r  > g ) *h = 240 + (120*(r-g)+delta)/(2*delta);
				else          *h = 180 + (120*(r-g+delta)+delta)/(2*delta);
				break;
		}
	}
}

void cN3::setRgb(int r, int g, int b)
{
	r = MAX (0, MIN (r, 255));
	g = MAX (0, MIN (g, 255));
	b = MAX (0, MIN (b, 255));
	value = getRGB (r,g,b);
}

void cN3::setHsv(int h, int s, int v)
{
	if (h < -1 || (UINT)s > 255 || (UINT)v > 255) return;

	int r(v), g(v), b(v);
	if ( s == 0 || h == -1 ) {
		;
	} else {
		if ((UINT)h >= 360) h %= 360;
		UINT f(h%60);
		h /= 60;
		UINT p = (UINT)(2*v*(255-s)+255)/510;
		UINT q, t;
		if (h & 1) {
			q = (UINT)(2*v*(15300-s*f)+15300)/30600;
			switch (h) {
				case 1: r=(int)q; g=(int)v, b=(int)p; break;
				case 3: r=(int)p; g=(int)q, b=(int)v; break;
				case 5: r=(int)v; g=(int)p, b=(int)q; break;
			}
		} else {
			t = (UINT)(2*v*(15300-(s*(60-f)))+15300)/30600;
			switch (h) {
				case 0: r=(int)v; g=(int)t, b=(int)p; break;
				case 2: r=(int)p; g=(int)v, b=(int)t; break;
				case 4: r=(int)t; g=(int)p, b=(int)v; break;
			}
		}
	}
	setRgb (r, g, b);
}

void cN3::setRgb(RGB rgb)
{
	int r = (int)( rgb        & 0xff);
	int g = (int)((rgb >>  8) & 0xff);
	int b = (int)((rgb >> 16) & 0xff);
	setRgb (r, g, b);
}

cN3 cN3::Light(int factor) const
{
	if (factor <= 0)       return *this;
	else if (factor < 100) return Dark(10000/factor);

	int h, s, v;
	hsv (&h, &s, &v);
	v = (factor*v)/100;
	if (v > 255) {
		s -= v-255;
		if ( s < 0 ) s = 0;
		v = 255;
	}

	return cN3(h, s, v, cN3::Hsv);
}

cN3 cN3::Dark(int factor) const
{
	if (factor <= 0)       return *this;
	else if (factor < 100) return Light(10000/factor);

	int h, s, v;
	hsv (&h, &s, &v);
	v = (v*100)/factor;

	return cN3(h, s, v, cN3::Hsv);
}

bool cN3::operator == (const cN3 &c) const
{
    return value == c.value;
}

bool cN3::operator != (const cN3 &c) const
{
    return value != c.value;
}

int cN3::operator [] (int i) const
{
    assert(0 <= i && i < 3);
    return i==0?red():(i==1?green():blue());
}

ostream & operator << (ostream &o, const cN3 &c)
{
	o << "c[" << c.red() << " " << c.green() << " " << c.blue() << "]" << endl; 
	return o;
}

//
// cR3 interface
//

cR3::cR3(double _r, double _g, double _b):r(_r),g(_g),b(_b){}

cR3::cR3(const cN3 &c):r(c.red()/255.),g(c.green()/255.),b(c.blue()/255.){}

cR3 cR3::operator + (const cR3 &c) const
{
	return cR3(r+c.r, g+c.g, b+c.b);
}

cR3 & cR3::operator += (const cR3 &c){
	r+=c.r; g+=c.g; b+=c.b;
	return *this;
}

cR3 cR3::operator - () const
{
	return cR3(-r,-g,-b);
}

cR3 cR3::operator - (const cR3 &c) const
{
	return cR3(r-c.r, g-c.g, b-c.b);
}

cR3 & cR3::operator -= (const cR3 &c){
	r-=c.r; g-=c.g; b-=c.b;
	return *this;
}

cR3 cR3::operator * (double k) const
{
	return cR3(r*k, g*k, b*k);
}

cR3 & cR3::operator *= (double k)
{
   r*=k; g*=k; b*=k;
   return *this;
}

cR3 cR3::operator * (const cR3 &c) const
{
	return cR3(r*c.r, g*c.g, b*c.b);
}

cR3 & cR3::operator *= (const cR3 &c)
{
   r*=c.r; g*=c.g; b*=c.b;
   return *this;
}

cR3 cR3::operator / (double k) const
{
	assert(k > EPS);
	return cR3(r/k, g/k, b/k);
}

cR3 & cR3::operator /= (double k)
{
	assert(k > EPS);
   r/=k; g/=k; b/=k;
   return *this;
}

cR3 cR3::operator / (const cR3 &c) const
{
	assert(c.r > EPS && c.g > EPS && c.b > EPS);
	return cR3(r/c.r, g/c.g, b/c.b);
}

cR3 & cR3::operator /= (const cR3 &c)
{
	assert(c.r > EPS && c.g > EPS && c.b > EPS);
   r/=c.r; g/=c.g; b/=c.b;
   return *this;
}

bool cR3::operator == (const cR3 &c) const
{
	return fabs(r-c.r)<=EPS && fabs(g-c.g)<=EPS && fabs(b-c.b)<=EPS;
}

bool cR3::operator != (const cR3 &c) const
{
	return !(*this == c);
}

double cR3::operator [] (int i) const
{
    assert(0 <= i && i < 3);
    return i==0?r:(i==1?g:b);
}

double & cR3::operator [] (int i)
{
    assert(0 <= i && i < 3);
    return i==0?r:(i==1?g:b);
}

double cR3::operator ! (void) const
{
	return SQ(r)+SQ(g)+SQ(b);
}

cR3 & cR3::Normalize()
{
	double n(!(*this));
	assert(n > EPS);
	*this /= sqrt(n);
	return *this;
}

void cR3::Clamp()
{
	r = MAX (0., MIN (r, 1.));
	g = MAX (0., MIN (g, 1.));
	b = MAX (0., MIN (b, 1.));
}

cR3 Normalize(const cR3 &c)
{
   double n(!c);
	assert(n > EPS);
	return c/sqrt(n);
}

ostream & operator << (ostream &o, const cR3 &c)
{
	o << "c[" << c.r << "," << c.g << "," << c.b << "]" << endl;
	return o;
}
