// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */

#include <stdarg.h>
#include <sstream>


#include <hud.hh>

GLvoid *font_style = GLUT_BITMAP_TIMES_ROMAN_10;

void
setfont(const char* name, int size)
{
    font_style = GLUT_BITMAP_HELVETICA_10;
    if (strcmp(name, "helvetica") == 0) {
        if (size == 12) 
            font_style = GLUT_BITMAP_HELVETICA_12;
        else if (size == 18)
            font_style = GLUT_BITMAP_HELVETICA_18;
    } else if (strcmp(name, "times roman") == 0) {
        font_style = GLUT_BITMAP_TIMES_ROMAN_10;
        if (size == 24)
            font_style = GLUT_BITMAP_TIMES_ROMAN_24;
    } else if (strcmp(name, "8x13") == 0) {
        font_style = GLUT_BITMAP_8_BY_13;
    } else if (strcmp(name, "9x15") == 0) {
        font_style = GLUT_BITMAP_9_BY_15;
    }
}

void 
drawstr(GLfloat x, GLfloat y, const char* format, ...)
{
    va_list args;
    char buffer[255], *s;
    
    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);
    
    glRasterPos3f(x, y,-1.5);
    for (s = buffer; *s; s++)
        glutBitmapCharacter(font_style, *s);
}



/*********************************/



HUD::HUD() : angleX(0.), lObj(), life(100.), armor(100.), helpActive(false)
{ 
	cam 	 = NULL;
	objActif = NULL;
	parts    = NULL;
	stock	 = new SAmoStock();
	bar	 = new HUD_Bar(this);

}

HUD::HUD(CameraFP* _c) :angleX(0.), lObj(),life(100.),armor(100.),helpActive(false)
{
	cam 	 = _c;
	objActif = NULL;
	parts    = NULL;
	stock	 = new SAmoStock();
	bar	 = new HUD_Bar(this);

}

HUD::~HUD()
{
	cam      = NULL;
	objActif = NULL;
	lObj.clear();
	
	if(parts)
		delete[] parts;
	delete bar, stock;
}



void HUD::addObject(SArme* se)
{
	if(se)
		lObj.push_back(se);
	if(lObj.size() == 1)
		setActif(0);
	
	parts = new Particule[PARTICULE_NB];
	
	for(int i(0);i<PARTICULE_NB;i++)
	{
		*(parts+i) = Particule(1.0, (float) (rand() % 100) / 1000 + 0.003f, NULL , Color(0.95, 0.67, 0.49));
		(parts+i)->setSpeed(vR3((float)((rand() % 50) - 26.0f) * 10.0f,(float) ((rand() % 50) - 25.0f) * 10.0f, (float)((rand() % 50) - 25.0f) * 10.0f));
		(parts+i)->setGravity(vR3(0., -0.05, 0.));
	}
}

void HUD::setActif(short ind)
{
	if(ind == 0)
		objActif = *(lObj.begin());
 	
	if( ind <  lObj.size() || ind<0)
	{
		if(ind==-1)
		{
			list<SArme*>::iterator it= find( lObj.begin(),lObj.end(),objActif );
			if( it!= lObj.end() )
			{
				it++;
				if( it!= lObj.end() )
					objActif= (*it);
				else objActif= *(lObj.begin());	
			}
			else objActif= *(lObj.begin());
		}
	}
	updatePosition();
}

void HUD::updatePosition()
{
//	static float last_ang = 0.;
	pR3 _p(cam->Oeil.x, cam->Oeil.y * 1.75, cam->Oeil.z + cam->Viser.z);
	if(objActif)
		objActif->updatePosition(pR3(0.,0.,0.));
		
//	last_ang = cam->Roty;
}

bool HUD::IsAlive()const
{ return life>0; }

void HUD::Reload()const
{
   if(objActif)
		switch( objActif->amo_used[0].getType() )
		{
			case (SAmo::NINE_MM): objActif->Reload(  stock->NB_NINE_MM,true );
					break;
				
			case (SAmo::PANZER): objActif->Reload(  stock->NB_PANZER,true);
					break;
						
			case (SAmo::C12): objActif->Reload(  stock->NB_C12 ,true);
					break;
			
			case (SAmo::GRENADE ): objActif->Reload(  stock->NB_GREN ,true);
            	    break;
			
			case (SAmo::P50): objActif->Reload(  stock->NB_P50 ,true  );
					break;
				
			case (SAmo::SEVEN65): objActif->Reload(  stock->NB_SEVEN65,true);
					break;
		}
 }



void HUD::setAction(bool is_active)
{

	if(objActif)
		switch( objActif->amo_used[0].getType() )
		{
			case (SAmo::NINE_MM): objActif->Fire(  stock->NB_NINE_MM,is_active );
					for(int i(0);i<PARTICULE_NB;i++)
						(parts+i)->explode();
					break;
				
			case (SAmo::PANZER): objActif->Fire(  stock->NB_PANZER,is_active);
					for(int i(0);i<PARTICULE_NB;i++)
						(parts+i)->explode();
					break;
						
			case (SAmo::C12): objActif->Fire(  stock->NB_C12 ,is_active);
					for(int i(0);i<PARTICULE_NB;i++)
						(parts+i)->explode();
					break;
			
			case (SAmo::GRENADE ):
                 cout<<"GRENADE"<<endl;
                 objActif->Fire(  stock->NB_GREN ,is_active);	break;
			
			case (SAmo::P50): objActif->Fire(  stock->NB_P50 ,is_active  );
					for(int i(0);i<PARTICULE_NB;i++)
						(parts+i)->explode();
					break;
				
			case (SAmo::SEVEN65): objActif->Fire(  stock->NB_SEVEN65,is_active);
					for(int i(0);i<PARTICULE_NB;i++)
						(parts+i)->explode();
					break;
		}
}


void HUD::updateAction(float degat)
{
	cout<<"armor_= "<<armor<<endl;
	cout<<"life_= "<<life<<endl;
	
	if(armor>0.)
	{
		float tmp( armor-(0.8*degat) );
		float tmp2( life-(0.2*degat) );
		
		if( tmp>=0. )
		{
			armor=tmp;
			if( tmp2>0 )
				life=tmp2;
			else
				life=0.;
		}
		else
		{
			tmp=armor, armor=0;
			if( (degat-tmp) < 0. )
				life=0.;
			else
				life-=(degat-tmp);
		}
	}
	else
		if( (life-degat) <=0. )
			life=0.;
		else
			life-=degat;
}

void HUD::displayHelp()
{
	glPushMatrix();
		glTranslatef(1.,1.3,0.);
		glColor3f(1., 1., 1.);
		setfont("helvetica", 18);
		drawstr(0.,0.,"Key  --- Command ");
		setfont("helvetica", 17);
		drawstr(0.,-0.1,"NUMPAD_0  --- Fire ");
		drawstr(0.,-0.15,"NUMPAD_8  --- Move up ");
		drawstr(0.,-0.2,"NUMPAD_2  --- Move down ");
		drawstr(0.,-0.25,"NUMPAD_4  --- Move left");
		drawstr(0.,-0.3,"NUMPAD_6  --- Move right");
		drawstr(0.,-0.35,"c  --- Change Weapon ");
		drawstr(0.,-0.4,"r  --- Reload ");
		drawstr(0.,-0.45,"h  --- Display/Hide help ");
		drawstr(0.,-0.5,"+ / -  --- Move up/down sound ");
		drawstr(0.,-0.55,"m  --- Mute ");
		drawstr(0.,-0.6,"f  --- Full screen ");
		drawstr(0.,-0.65,"Echap  --- Quit ");
	glPopMatrix();


}

		
void HUD::display()
{
	double rad2deg(180. / PI);
	static int nb_sec = 0;
	glEnable(GL_BLEND);
	
	if(angleX > 10.)
		objActif->inActionUp = false, objActif->inActionDown = true;
	
	GLfloat angle(acos(cam->ViserZ%vR3(0.,0.,-1.)) * rad2deg);
	glPushMatrix();
		glTranslatef(cam->Oeil.x, cam->Oeil.y, cam->Oeil.z);
		glTranslatef(cam->Viser.x, cam->Viser.y, cam->Viser.z);
		glRotatef(-cam->Roty*rad2deg,0.,1.,0.);
		glRotatef( ( cam->ViserZ.y > 0 ? angle : -angle ), 1.,0., 0. );
		glColor3f( 1., 1., 1. );
		glBegin(GL_LINES);
			glVertex3f(-.03,0.,-1.);
		 	glVertex3f(-.01,0.,-1.);
			glVertex3f(.01,0.,-1.);
			glVertex3f(.03,0.,-1.);
			glVertex3f(0.,.01,-1.);
			glVertex3f(0.,.03,-1.);
			glVertex3f(0.,-.03,-1.);
			glVertex3f(0.,-.01,-1.);
		glEnd();
		if( bar )
			bar->display();

		if(objActif->fire && nb_sec < 5)
		{
			++nb_sec;
           if( objActif->getArmeType() != SArme::GRENADE )
           {
			   glPushMatrix();
			      glTranslatef(-4., 0., 0.);
				  for(int i(0);i<PARTICULE_NB;i++)
					(parts+i)->display();
			    glPopMatrix();
           }     
		}
		else
		{
			nb_sec = 0;
			objActif->fire   = false;
		}
	
		for(int i(0);i<PARTICULE_NB;i++)
			(parts+i)->update(1., Color(0.75, 0.47, 0.29));
		
		
		if(helpActive)
		{
			HUD::displayHelp();
		}
		
		
		glTranslatef(-.8, -0.5, -0.6);

		if(objActif->inActionUp)
			glRotatef(angleX+=3., 1., 0., 0.);
		else
		{
			if(angleX > 0. && objActif->inActionDown)
				glRotatef(angleX-=10., 1., 0., 0);
		}
		if(objActif)
			objActif->display();
	
	glPopMatrix();
}


void HUD::setHelpActif(bool act)
{	helpActive=act;	}

bool HUD::getHelpActif() const
{	return helpActive;	}

SArme::SArmeType HUD::getArmeType() const
{
	if(objActif)
		return objActif->getArmeType();
	else
		return SArme::UNDEFINED;
}


HUD_Bar::HUD_Bar(HUD* per):Perso(per)
{
}
	
	
HUD_Bar::~HUD_Bar()
{
	if( !Perso )
		Perso=NULL;
}
	
void HUD_Bar::display()
{
	GLfloat bla[3]= {0.,1.,0.};
	GLfloat gri[3]= {0.4,0.4,0.4};
	
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		
	glPushMatrix();
	
		glTranslatef(0.7,-1.2,0.);

		glPushMatrix();
			glColor3f(0.3,0.3,0.3);	
			glBegin( GL_POLYGON );
				glVertex3f(0.05,0.1,-1.08);
				glVertex3f( 0.1 ,0.05,-1.08);
				glVertex3f(0.4,0.05,-1.08);
				glVertex3f(0.45,0.1,-1.08);
				
				glVertex3f(0.45,0.25,-1.08);
				glVertex3f( 0.4 ,0.3,-1.08);
				glVertex3f(0.1,0.3,-1.08);
				glVertex3f(0.05,0.25,-1.08);
				glVertex3f(0.05,0.1,-1.08);
				
				
			glEnd();
			
			/*  Petites facettes */
			glColor3f(0.4,0.4,0.4);
			
			glBegin( GL_POLYGON );
				glVertex3f(0.0,0.05,-1.1);
				glVertex3f( 0.05 ,0.,-1.1);
				glVertex3f( 0.1 ,0.05,-1.08);
				glVertex3f(0.05,0.1,-1.08);
				glVertex3f(0.0,0.05,-1.1);
			glEnd();
			
			glBegin( GL_POLYGON );
				glVertex3f(0.45,0.,-1.1);
				glVertex3f(0.5,0.05,-1.1);
				glVertex3f(0.45,0.1,-1.08);
				glVertex3f(0.4,0.05,-1.08);
				glVertex3f(0.45,0.,-1.1);
			glEnd();
			
			glBegin( GL_POLYGON );			
				glVertex3f(0.5,0.3,-1.1);
				glVertex3f( 0.45 ,0.35,-1.1);
				glVertex3f( 0.4 ,0.3,-1.08);
				glVertex3f(0.45,0.25,-1.08);
				glVertex3f(0.5,0.3,-1.1);
			glEnd();

			glBegin( GL_POLYGON );
				glVertex3f(0.05,0.35,-1.1);
				glVertex3f(0.1,0.3,-1.08);
				glVertex3f(0.05,0.25,-1.08);
				glVertex3f(0.,0.3,-1.1);
				glVertex3f(0.05,0.35,-1.1);
			glEnd();
	
			/*---------------*/
			
			
			
			/* Grandes facettes */
			glColor3f(0.5,0.5,0.5);
			glBegin( GL_POLYGON );
				glVertex3f( 0.05 ,0.,-1.1);
				glVertex3f(0.45,0.,-1.1);
				glVertex3f(0.4,0.05,-1.08);
				glVertex3f( 0.1 ,0.05,-1.08);
				glVertex3f( 0.05 ,0.,-1.1);
			glEnd();
			
			
			
			glBegin( GL_POLYGON );
				glVertex3f(0.45,0.1,-1.08);
				glVertex3f(0.5,0.05,-1.1);
				glVertex3f(0.5,0.3,-1.1);
				glVertex3f(0.45,0.25,-1.08);
				glVertex3f(0.45,0.1,-1.08);
			glEnd();			
			
			
			glBegin( GL_POLYGON );
				glVertex3f( 0.45 ,0.35,-1.1);
				glVertex3f( 0.4 ,0.3,-1.08);
				glVertex3f(0.1,0.3,-1.08);	
				glVertex3f(0.05,0.35,-1.1);
				glVertex3f( 0.45 ,0.35,-1.1);
			glEnd();


			
			glBegin( GL_POLYGON );
				glVertex3f(0.,0.3,-1.1);
				glVertex3f(0.05,0.25,-1.08);
				glVertex3f(0.05,0.1,-1.08);
				glVertex3f(0.0,0.05,-1.1);
				glVertex3f(0.,0.3,-1.1);
				
			glEnd();
			/************/

		glPopMatrix();
		
		
		
		
		
		/*   Barre de vie */
		glPushMatrix();
			glTranslatef(0.1,0.21,0.);
			
			glPushMatrix();
						
				glColor3f(0.0,1.,0.);	
				
				glBegin( GL_POLYGON );
					glVertex3f(0.,0.,-1.079);
					glVertex3f( (( 0.3 * Perso->life )/100 ) ,0.,-1.079);
					glVertex3f((( 0.3 * Perso->life )/100 ),0.05,-1.079);
					glVertex3f(0.,0.05,-1.079);
				glEnd();
			
			
				glColor3f(0.,0.,0.);	
				glBegin( GL_POLYGON );
					glVertex3f( (( 0.3 * Perso->life )/100 ) ,0.,-1.079);
					glVertex3f( 0.3 ,0.,-1.079);
					glVertex3f( 0.3,0.05,-1.079);
					glVertex3f( (( 0.3 * Perso->life )/100 ) ,0.05,-1.079);
				glEnd();
			glPopMatrix();
	
		glPopMatrix();
		/***********************/
		
		
		
		/*   Barre d armure */
		glPushMatrix();
			glTranslatef(0.1,0.1,0.);
			
			glPushMatrix();
						
				glColor3f(0.,0.,1.);	
				glBegin( GL_POLYGON );
					glVertex3f(0.,0.,-1.079);
					glVertex3f( (( 0.3 * Perso->armor )/100 ) ,0.,-1.079);
					glVertex3f((( 0.3 * Perso->armor )/100 ),0.05,-1.079);
					glVertex3f(0.,0.05,-1.079);
				glEnd();
			
			
				glColor3f(0.,0.,0.);	
				glBegin( GL_POLYGON );
					glVertex3f( (( 0.3 * Perso->armor )/100 ) ,0.,-1.079);
					glVertex3f( 0.3 ,0.,-1.079);
					glVertex3f( 0.3,0.05,-1.079);
					glVertex3f( (( 0.3 * Perso->armor )/100 ) ,0.05,-1.079);
				glEnd();
			glPopMatrix();
	
		glPopMatrix();
		/***********************/
		
		glEnable(GL_LIGHTING);
		glBlendFunc(GL_ONE, GL_ZERO);
	
		if(Perso->life==0)
		{	glDisable(GL_LIGHTING);
			glPushMatrix();
				//glLoadIdentity();
				glTranslatef(-1.,1.,0.);
				glColor3f(1., 1., 1.);
				setfont("helvetica", 30);
				drawstr(0.,0.,"Vous etes mort ....");
			glPopMatrix();
		}
		else
		{

			/* Arme */
			if( Perso->objActif )
			{
				stringstream nb_tmp,nb_tmp2;

				if(Perso->objActif)
				{	
					nb_tmp<< ((Perso->objActif)->getNbAmo());
					switch( Perso->objActif->amo_used[0].getType() )
					{
						case (SAmo::NINE_MM): nb_tmp2<< ((Perso->stock)->NB_NINE_MM); break;
				
						case (SAmo::PANZER): nb_tmp2<< ((Perso->stock)->NB_PANZER); break;
					
						case (SAmo::C12): nb_tmp2<< ((Perso->stock)->NB_C12); break;
			
						case (SAmo::GRENADE ): nb_tmp2<< ((Perso->stock)->NB_GREN); break;
			
						case (SAmo::P50): nb_tmp2<< ((Perso->stock)->NB_P50); break;
				
						case (SAmo::SEVEN65): nb_tmp2<< ((Perso->stock)->NB_SEVEN65); break;
					}
				}
			
				glTranslatef(0.95,0.,-1.5);
				glPushMatrix();
					glTranslatef(0.8,-1.,0.);
					glDisable(GL_LIGHTING);
					glColor3f(1., 1., 1.);
					setfont("helvetica", 18);
				
					drawstr(0.,0.,(char*) nb_tmp.str().data());
					drawstr(0.1,0.,"  /  ");
				
				
					drawstr(0.2,0., (char*) nb_tmp2.str().data() );
					glEnable(GL_LIGHTING);
				
				glPopMatrix();
			
				glRotatef(80.,0.,1.,0.);
				glRotatef(30.,0.,0.,1.);
				glScalef(0.3,0.3,0.3);
				Perso->objActif->display();
			
			
			}
		}
		/***************/
		
	glPopMatrix();
	
}

void HUD::writeMsg(const char* msg, const Color& col)
{
	glPushMatrix();
		//glTranslatef(0.7, -1.2, -0.8);
		glColor3f(1., 1., 1.);
		setfont("helvetica", 12);
		drawstr(0., 0., msg);
	glPopMatrix();	
}
		

	
void HUD_Bar::update()
{

}
