// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <motor/interaction.hh>
#include <motor/collision.hh>

using namespace std;
using namespace mCG;

/* -------------------- Class Interaction Methods -------------------- */

SInteraction::SInteraction(SObject** _o) : sTime(0.)
{
	sObj = _o;
	delta_t = 0.;
	sEntity = NULL;
}

SInteraction::SInteraction(SEntity** obj) : sTime(0.), delta_t(0.)
{
	assert(obj);
	sObj = NULL;
	sEntity = obj;
}

void SInteraction::setObj(SObject** _o)
{
	sTime=0.;
	sObj = _o;
	delta_t = 0.;
	sEntity = NULL;
} 
	
void SInteraction::init()
{	
	if(!sObj && sEntity)
		sObj = (SObject**)&((*sEntity)->boundBox);
	
	if(sObj && *sObj)
	{	
		sVt = (*sObj)->getV0();
		sPt = (*sObj)->getMassCenter();
		sP0 = sPt;
	}

	sTime = 0.;

	delta_t = 1. / sqrt(!sVt);

	if(delta_t > DELTA_T)
		delta_t = DELTA_T;
	
	(*sObj)->setAnimated(true);
}


pR3 SInteraction::getCenter() const
{
	return sPt;
}

vR3 SInteraction::getSpeed() const
{
	return sVt;
}

double SInteraction::getExecTime() const
{
	return sTime;
}

double SInteraction::getDeltaTime() const
{
	return delta_t;
}

SObject* SInteraction::onAction(const list<SObject*> lObject, bool& inter_or_not, bool param)
{
	bool result(false);
	
	inter_or_not   = false;
	SObject* ptRes = NULL;
	SCollision *collision_detect = new SPlanBox;
	list<SObject*>::const_iterator it_obj = lObject.begin();
	
		if(sObj && (*sObj)->isAnimated())
		{
			
			vR3 vWeight;
		
			unsigned int nb_contact(0);
			double alpha(0.);
			list<SPlan>::const_iterator it;

			sTime += delta_t;

			/* - Local integration - */

			for(double nbb=0.;nbb<DELTA_T;nbb+=delta_t)
			{
				/* - Using Euler approximation function to calculate the integrals -*/

				if(!((*sObj)->hasSupport()))
					vWeight = G_VECT;
				else
				{
					double alphax = acos((vR3(1.,0.,0.) % (*sObj)->getSupportDirection()) / (sqrt(!vR3(1.,0.,0.)) *
						sqrt(!((*sObj)->getSupportDirection()))));
					double alphay = acos((vR3(0.,1.,0.) % (*sObj)->getSupportDirection()) / (sqrt(!vR3(0.,1.,0.)) *
						sqrt(!((*sObj)->getSupportDirection()))));
					double alphaz = acos((vR3(0.,0.,1.) % (*sObj)->getSupportDirection()) / (sqrt(!vR3(0.,0.,1.)) *
						sqrt(!((*sObj)->getSupportDirection()))));
					vWeight = vR3( (*sObj)->mass * -G_CONST * cos(alphax), (*sObj)->mass * -G_CONST * sin(alphay),
							(*sObj)->mass * -G_CONST * -cos(alphaz) );
				}
			}
	
		for(;it_obj != lObject.end();it_obj++)
		{
			if((*it_obj) != *sObj && collision_detect->collisionDetected(*this, *((SBox*)(*it_obj))))
			{
				result = true;

				vR3 vx = vR3(sVt.x,0.,0.);
				vR3 vy = vR3(0.,sVt.y,0.);
				vR3 vz = vR3(0.,0.,sVt.z);

				vR3 dist(sPt - (*it_obj)->getMassCenter());
				vR3 _n(dist.Normalize());

				if(!param)
					sVt.y *= (*sObj)->density;
				else
				{
					if((vx%_n) < 0.)
						sVt.x *= -1.;
					if((vy%_n) < 0.)
						sVt.y *= -1.;
					if((vz%_n) < 0.)
						sVt.z *= -1.;
					
					sVt *= (*sObj)->density;
				}
				
				ptRes = (*it_obj);
				
				break;
			}
		}
		
		inter_or_not = result;
		
		sVt = sVt + vWeight * delta_t;
		sPt = sPt + sVt * delta_t;

		(*sObj)->setV0(sVt);
		(*sObj)->updatePosition(sPt);
		
		if(sEntity)
		{
			(*sEntity)->setV0(sVt);
			
			if(param)
				(*sEntity)->setInitPos(sPt);
			else
				(*sEntity)->updatePosition(sPt);
		}
	}
	
	if(collision_detect)
	{
		delete collision_detect;
		collision_detect = NULL;
	}
	
	return ptRes;
}

///////////////////////////////////////////////// ENTITY ONE ////////////////////////////////////////////////////
