// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <loader.hh>
#include <motor/real_obj.hh>

////////////////////////////////////////////////////////////////

loaderPLY::loaderPLY(SPolyhedron** _o)
{
	obj = _o;
}

loaderPLY::~loaderPLY()
{
	obj = NULL;
}

bool loaderPLY::load(const char* fic)
{
	assert(fic);
	assert(obj);
			
	unsigned nbsom(0), nbface(0), spf;
	ifstream flit(fic);
	pR3 tmp;
	int a, b, c;
	float fa, fb;
	
	flit >> nbsom;
	flit >> nbface;
		
	for(int i(nbsom);i--;)
	{
		flit >> tmp.x >> tmp.y >> tmp.z;
		(*obj)->sVList.push_back(tmp);
	}
	
	for(unsigned i(0);i<nbface*3;i+=3)
	{
		flit >> spf;
		flit >> a >> b >> c;
		(*obj)->sFList.push_back(Face(a, b, c));
	}
	
	((SPerso*)(*obj))->allocTexture(nbface*3);
	
	for(unsigned i(0);i<nbsom*2;i+=2)
	{
		
		flit >> fa >> fb;
		
		((SPerso*)(*obj))->setTexture(fa, i);
		((SPerso*)(*obj))->setTexture(fb, i+1);
	}
	
	flit.close();
	
	return true;
}


////////////////////////////////////////////////////////////////

loaderMAX::loaderMAX(SEntity** _ent)
{
	enti = _ent;
}

loaderMAX::~loaderMAX()
{
	enti = NULL;
}

void loaderMAX::readMaterials(ifstream& flit, long len, vector<Color>& lColor)
{
	bool done(false);
	bool is_diffuse(false), is_specular(false), is_ambiant(false);
	unsigned long pptmp = flit.tellg();
	unsigned long count(pptmp + (len - 6));
	unsigned long pos;
	USHORT id;
	UINT leng;
	char c;

	unsigned char r, g, b;
	
	while(!done)
	{
		flit.read((char*)&id, sizeof(USHORT));
		if(flit.eof())
		{
			done = true;
			break;
		}
		
		flit.read((char*)&leng, sizeof(UINT));
		
		switch(id)
		{ 
			case 0xA000: do { flit.read((char*)&c, sizeof(char)); }while(c != 0); break;
			case 0xA010: is_ambiant = true; is_diffuse = false; is_specular = false; break;
			case 0xA020: is_diffuse = true; is_ambiant = false; is_specular = false; break;
			case 0xA030: is_specular= true; is_diffuse = false; is_ambiant = false;  break;
			case 0x0011: if(is_ambiant)
				     {
					flit.read((char*)&r, sizeof(unsigned char));
				     	flit.read((char*)&g, sizeof(unsigned char)); 
			             	flit.read((char*)&b, sizeof(unsigned char));
					lColor.push_back(Color(((double)r) / 255., ((double)g) / 255., ((double)b) / 255.));
				     }
				     else
					    lColor.push_back(Color(-1., -1., -1.)); 
				     if(is_diffuse)
				     {
					flit.read((char*)&r, sizeof(unsigned char));
				     	flit.read((char*)&g, sizeof(unsigned char)); 
			             	flit.read((char*)&b, sizeof(unsigned char));
					lColor.push_back(Color(((double)r) / 255., ((double)g) / 255., ((double)b) / 255.));
				     }
				     else
					    lColor.push_back(Color(-1., -1., -1.));
				     if(is_specular)
				     {
					flit.read((char*)&r, sizeof(unsigned char));
				     	flit.read((char*)&g, sizeof(unsigned char)); 
			             	flit.read((char*)&b, sizeof(unsigned char));
					lColor.push_back(Color(((double)r) / 255., ((double)g) / 255., ((double)b) / 255.));
				     }
				     else
					    lColor.push_back(Color(-1., -1., -1.));
				     break;
				     
			default: pos = flit.tellg();
        		if((pos - 6) >= count)
        		{
        		  	flit.seekg(-6, ios::cur);
        		  	done = 1;
        		  	break;
        		}
        		pos += leng-6;
        		if(flit.seekg(pos, ios::beg)) done = 1;
			break;
		}
	}
}

bool loaderMAX::load(const char* fic)
{
	assert(enti);
			
	USHORT id, l_qty(0);
	UINT leng;
	UCHAR l_char;
	float pTmp1,pTmp2, pTmp3;
	USHORT  valT, valT1, valT2, valT3;
	ifstream flit(fic, ios::binary | ios::in);
	
	SPerso* new_obj = NULL;
	
	bool passe(false);
	vector<Color> lColor;
	
	while(!flit.eof())
	{
		l_char = 'n';
				
		flit.read((char*)&id, sizeof(USHORT));
		flit.read((char*)&leng, sizeof(UINT));

		UINT ind(0);
		Color amb(-1., -1., -1.), dif(-1., -1., -1.), spe(-1., -1., -1.);
		
		switch(id)
		{
			/*case 0xAFFF: readMaterials(flit,leng, lColor); 
			break;*/
			case 0x4000:
				for(int i(0);i<20 && l_char != '\0';i++)
					flit.read((char*)&l_char, sizeof(UCHAR));break;
				
			case 0x4110: 	passe = true; new_obj = new SPerso;
					flit.read((char*)&l_qty, sizeof(USHORT));

					for(USHORT s(0);s<l_qty;s++)
					{
						flit.read((char*)&(pTmp1), sizeof(float));
						flit.read((char*)&(pTmp2), sizeof(float));
						flit.read((char*)&(pTmp3), sizeof(float));

						new_obj->sVList.push_back(SPoint(pTmp1, pTmp2, pTmp3));
					}

					break;
				
			case 0x4120: 		
						flit.read((char*)&l_qty, sizeof(USHORT));

						for(USHORT s(0);s<l_qty;s++, ind+=3)
						{
							flit.read((char*)&valT1, sizeof(USHORT));
							flit.read((char*)&valT2, sizeof(USHORT));
							flit.read((char*)&valT3, sizeof(USHORT));
							flit.read((char*)&valT, sizeof(USHORT));
							
							new_obj->sFList.push_back(Face(valT1, valT2, valT3));
						} 

						break;
			
			case 0x4d4d: break;
			case 0x3d3d: break;
			case 0x4140: 	 	flit.read((char*)&l_qty, sizeof(USHORT));
						new_obj->allocTexture((unsigned)l_qty*2);
						
						for(USHORT s(0);s<l_qty;s++, ind+=2)
						{
							flit.read((char*)(&pTmp1), sizeof(float)); 
							flit.read((char*)(&pTmp2), sizeof(float));
							new_obj->setTexture(-pTmp1, ind);
							new_obj->setTexture(-pTmp2, ind+1);
						}
						break;
			case 0x4100: break;
			
			default: flit.seekg(leng-6, ios::cur); break;
		}
		
		if(passe)
		{
			(*enti)->allObj.push_back(new_obj);
			passe = false;
		}
	}

	flit.close();
	
	return true;
}

/*
bool loaderMd2::load(const char* fic)
{
	
}*/
		
Md2Model::Md2Model()
{
	m_pSkins = NULL;
	m_pTexCoords = NULL;
	m_pTriangles  = NULL;
	m_pFrames = NULL;
	m_pGLcmds = NULL;
	
	m_fScale = 0.5f;
}

Md2Model::~Md2Model()
{
	if(m_pSkins)
		delete m_pSkins;
	if(m_pTexCoords)
		delete m_pTexCoords;
	if(m_pTriangles)
		delete m_pTriangles;
	if(m_pFrames)
		delete m_pFrames;
	if(m_pGLcmds)
		delete m_pGLcmds;
}
		
bool Md2Model::loadModel(const char* fic)
{
	fstream    file;

	file.open(fic, std::ios::in | std::ios::binary);

	if(file.fail())
            return false;

	// lecture du header
	file.read((char *)&m_kHeader, sizeof(Md2Header));

	// vrification de l'authenticit du modle
	if((m_kHeader.version != MD2_VERSION) || m_kHeader.ident != MD2_IDENT)
		return false;

	// allocation de mmoire pour les donnes du modle
	m_pSkins     = new skinMd2[m_kHeader.num_skins];
	m_pTexCoords = new texMd2[m_kHeader.num_st];
	m_pTriangles = new triangMd2[m_kHeader.num_tris];
	m_pFrames    = new frameMd2[m_kHeader.num_frames];
	m_pGLcmds    = new int[m_kHeader.num_glcmds];

	// lecture des noms de skins
	file.seekg( m_kHeader.offset_skins, std::ios::beg );
	file.read( (char *)m_pSkins, sizeof( char ) * 68 * m_kHeader.num_skins );
	
	//for(int i(0);i<m_kHeader.num_skins;i++)
		//cout << m_pSkins[i].name << endl;
	
	
	// lecture des coordonnes de texture
	file.seekg( m_kHeader.offset_st, std::ios::beg );
	file.read( (char *)m_pTexCoords, sizeof( texMd2 ) * m_kHeader.num_st );

	
	
	// lecture des triangles
	file.seekg(m_kHeader.offset_tris, std::ios::beg);
	file.read((char *)m_pTriangles, sizeof(triangMd2) * m_kHeader.num_tris);
	
	cout << "RR: " << m_kHeader.num_st << endl;
	cout << "RR: " << m_kHeader.num_vertices << endl;
	cout << "RR: " << m_kHeader.num_tris << endl;

	// lecture des frames
	file.seekg(m_kHeader.offset_frames, std::ios::beg);

	for( int i = 0; i < m_kHeader.num_frames; i++ )
	{
            // allocation de mmoire pour les vertices
            m_pFrames[i].verts = new vertMd2[ m_kHeader.num_vertices ];

            // lecture des donnes de la frame
            file.read((char *)&m_pFrames[i].scale, sizeof(vec3_t));
            file.read((char *)&m_pFrames[i].translate, sizeof(vec3_t));
            file.read((char *)&m_pFrames[i].name, sizeof(char) * 16);
            file.read((char *)m_pFrames[i].verts, sizeof(vertMd2) * m_kHeader.num_vertices);
	}

	// lecture des commandes opengl
	file.seekg(m_kHeader.offset_glcmds, std::ios::beg);
	file.read((char *)m_pGLcmds, sizeof(int) * m_kHeader.num_glcmds);

	file.close();
	
	return true;
}

void Md2Model::display()
{
	int iMaxFrame = m_kHeader.num_frames - 1;
	
	int iFrame = 0;

    // vrification de la validit de iFrame
    if( (iFrame < 0) || (iFrame > iMaxFrame) )
        return;


    // activation de la texture du modle
  //  glBindTexture( GL_TEXTURE_2D, m_uiTexID );

    // dessin du modle
    glBegin( GL_POINTS );
    glColor3f(1., 0., 0.);
        // dessine chaque triangle
     /*   for( int i = 0; i < m_kHeader.num_tris; i++ )
        {
            // dessigne chaque vertex du triangle
            for( int k = 0; k < 3; k++ )
            {
                frameMd2 *pFrame = &m_pFrames[ iFrame ];
                vertMd2  *pVert  = &pFrame->verts[ m_pTriangles[i].vertex[k] ];


                // [coordonnes de texture]
                GLfloat s = (GLfloat)m_pTexCoords[ m_pTriangles[i].st[k] ].s / m_kHeader.skinwidth;
                GLfloat t = (GLfloat)m_pTexCoords[ m_pTriangles[i].st[k] ].t / m_kHeader.skinheight;

                // application des coordonnes de texture
                glTexCoord2f( s, t );

                // [normale]
               // glNormal3fv( m_kAnorms[ pVert->normalIndex ] );

                // [vertex]
                vec3_t v;

                // calcul de la position absolue du vertex et redimensionnement
                v.x = (pFrame->scale.x * pVert->v[0] + pFrame->translate.x) * m_fScale;
                v.y = (pFrame->scale.y * pVert->v[1] + pFrame->translate.y) * m_fScale;
                v.z = (pFrame->scale.z * pVert->v[2] + pFrame->translate.z) * m_fScale;

		GLfloat tab_tmp[3];
		memcpy(tab_tmp, &v, sizeof(float)*3);
		
                glVertex3fv( tab_tmp );
            }
        }*/
			
	frameMd2 *pFrame = &m_pFrames[ iFrame ];
	
	for( int i = 0; i < m_kHeader.num_vertices; i++ )
        {
            // dessigne chaque vertex du triangle
            
                
                vertMd2  *pVert  = &pFrame->verts[i];


                // [coordonnes de texture]
              //  GLfloat s = (GLfloat)m_pTexCoords[ m_pTriangles[i].st[k] ].s / m_kHeader.skinwidth;
             //   GLfloat t = (GLfloat)m_pTexCoords[ m_pTriangles[i].st[k] ].t / m_kHeader.skinheight;

                // application des coordonnes de texture
              //  glTexCoord2f( s, t );

                // [normale]
               // glNormal3fv( m_kAnorms[ pVert->normalIndex ] );

                // [vertex]
                vec3_t v;

                // calcul de la position absolue du vertex et redimensionnement
                v.x = (pFrame->scale.x * pVert->v[0] + pFrame->translate.x) * m_fScale;
                v.y = (pFrame->scale.y * pVert->v[1] + pFrame->translate.y) * m_fScale;
                v.z = (pFrame->scale.z * pVert->v[2] + pFrame->translate.z) * m_fScale;

		GLfloat tab_tmp[3];
		memcpy(tab_tmp, &v, sizeof(float)*3);
		
                glVertex3fv( tab_tmp );
            }		
	
    glEnd();
}

void Md2Model::setNormals(const char* fic_name)
{
	ifstream flit(fic_name, ios::in);
	
	vec3_t vecteur;
	
	vecteur.x = 0;
	vecteur.y = 0;
	vecteur.z = 0;
	
	//flit.read((char*)m_kAnorms, sizeof(float)*3*NUMVERTEXNORMALS);
	/*flit >> vecteur.x >> vecteur.y >> vecteur.z;
	flit >> vecteur.x >> vecteur.y >> vecteur.z;*/
			
	for(int i(0);i<NUMVERTEXNORMALS;i++)
		flit >> m_kAnorms[i].x >> m_kAnorms[i].y >> m_kAnorms[i].z;
	
	cout << m_kAnorms[0].x << ", " << m_kAnorms[0].y << ", " << m_kAnorms[0].z << endl;
	cout << m_kAnorms[1].x << ", " << m_kAnorms[1].y << ", " << m_kAnorms[1].z << endl;
	
	flit.close();
}
