// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */

#include <loadppm.hh>

PPMImage::PPMImage(const char *filename)
{
    char buff[16];

    ifstream fp(filename, ios::in|ios::binary);
    int maxval,lig_comm=0;

    if (!fp.is_open()){
	cerr <<"Unable to open file " << filename << endl;
	throw 0;
    }

    fp.read(buff, 3);
    if (fp.bad()){
      cerr <<"Unable to open file " << filename << endl;
      throw 0;
    }
    
    if (buff[0] != 'P' || buff[1] != '6'){
	cerr << "Invalid image format (must be `P6')\n" ;
	throw 0;
    }

    if (buff[2] != 0x0a)
      fp.seekg(-1,ios::cur);

    while (fp.read(buff, 1), buff[0] == '#'){
      lig_comm++;
      while (fp.read(buff, 1), buff[0] != '\n');
    }

    fp.seekg(-1,ios::cur);

    fp >> sizeX;
    fp >> sizeY;
    if (fp.bad()){
      cerr << "Error loading image " << filename << endl ;
      throw 0;
    }

    fp >> maxval;
    if (fp.bad()){
      cerr << "Error loading image " << filename << endl ;
      throw 0;
    }

    while (fp.read(buff, 1), buff[0] != '\n')
      ;

    data = new  char[3 * sizeX * sizeY];
    if (!data)
    {
      cerr <<  "Unable to allocate memory\n";
      throw 0;
    }

    fp.read(data, 3 * sizeX*sizeY);
    if (fp.bad()) {
      cerr <<  "Error loading image " << filename << endl;
      throw 0;
    }
    
    fp.close();
}

int loadBMP(char *filename, textureImage *texture)
{
    FILE *file;
    unsigned short int bfType;
    long int bfOffBits;
    short int biPlanes;
    short int biBitCount;
    long int biSizeImage;
    int i;
    unsigned char temp;

    if ((file = fopen(filename, "rb")) == NULL)
    {
        printf("File not found : %s\n", filename);
        return 0;
    }
    if(!fread(&bfType, sizeof(short int), 1, file))
    {
        printf("Error reading file!\n");
        return 0;
    }

    if (bfType != 19778)
    {
        printf("Not a Bitmap-File!\n");
        return 0;
    }        
    
    fseek(file, 8, SEEK_CUR);
    /* get the position of the actual bitmap data */
    if (!fread(&bfOffBits, sizeof(long int), 1, file))
    {
        printf("Error reading file!\n");
        return 0;
    }
    /* skip size of bitmap info header */
    fseek(file, 4, SEEK_CUR);
    /* get the width of the bitmap */
    fread(&texture->width, sizeof(int), 1, file);
    /* get the height of the bitmap */
    fread(&texture->height, sizeof(int), 1, file);
 
    /* get the number of planes (must be set to 1) */
    fread(&biPlanes, sizeof(short int), 1, file);
    if (biPlanes != 1)
    {
        printf("Error: number of Planes not 1!\n");
        return 0;
    }
    /* get the number of bits per pixel */
    if (!fread(&biBitCount, sizeof(short int), 1, file))
    {
        printf("Error reading file!\n");
        return 0;
    }
    if (biBitCount != 24)
    {
        printf("Bits per Pixel not 24\n");
        return 0;
    }
    /* calculate the size of the image in bytes */
    biSizeImage = texture->width * texture->height * 3;
    texture->data = (unsigned char*)malloc(biSizeImage);
    /* seek to the actual data */
    fseek(file, bfOffBits, SEEK_SET);
    if (!fread(texture->data, biSizeImage, 1, file))
    {
        printf("Error loading file!\n");
        return 0;
    }
    /* swap red and blue (bgr -> rgb) */
    for (i = 0; i < biSizeImage; i += 3)
    {
        temp = texture->data[i];
        texture->data[i] = texture->data[i + 2];
        texture->data[i + 2] = temp;
    } 
    return 1;
}
