// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <motor/lumiere.hh>

using namespace mCG;

/////////////// LUMIERE (abstraite) //////////////////////////////////

Lumiere::Lumiere() : pos(), couleur()
{
	num++;
	
	switch(num)
	{
		case 0: lightType = GL_LIGHT0; break; 
		case 1: lightType = GL_LIGHT1; break;
		case 2: lightType = GL_LIGHT2; break;
		case 3: lightType = GL_LIGHT3; break;
		case 4: lightType = GL_LIGHT4; break;
		case 5: lightType = GL_LIGHT5; break;
		case 6: lightType = GL_LIGHT6; break;
		default: break;
	}
}
	
Lumiere::Lumiere(const pR3& p, Color* c) : pos(p)
{
	memcpy(couleur, c, sizeof(Color)*3);
	
	++num;
	
	switch(num)
	{
		case 0: lightType = GL_LIGHT0; break; 
		case 1: lightType = GL_LIGHT1; break;
		case 2: lightType = GL_LIGHT2; break;
		case 3: lightType = GL_LIGHT3; break;
		case 4: lightType = GL_LIGHT4; break;
		case 5: lightType = GL_LIGHT5; break;
		case 6: lightType = GL_LIGHT6; break;
		default: break;
	}
	
	GLfloat tab1[3] = { couleur[0][0], couleur[0][1], couleur[0][2] };
	GLfloat tab2[3] = { couleur[1][0], couleur[1][1], couleur[1][2] };
	GLfloat tab3[3] = { couleur[2][0], couleur[2][1], couleur[2][2] };
	
	//glLightfv(lightType, GL_AMBIENT,  tab1);
	glLightfv(lightType, GL_DIFFUSE,  tab2);
	glLightfv(lightType, GL_SPECULAR, tab3);
}
	
Lumiere::~Lumiere()
{
	--num;
}

int Lumiere::num = -1;

void Lumiere::enable() const
{
	glEnable(lightType);
}

void Lumiere::disable() const
{
	glDisable(lightType);
}

void Lumiere::setPos(const pR3& _p)
{
	pos = _p;
}

const pR3& Lumiere::getPos() const
{
	return pos;
}

////////////// LUMIERE DIRECTIONNELLE ////////////////////////////////


LumiereDir::LumiereDir() : Lumiere(), dir()
{}
	
LumiereDir::LumiereDir(const pR3& p, Color* c, const vR3& v) : Lumiere(p, c), dir(v)
{}
	
LumiereDir::~LumiereDir()
{}

void LumiereDir::init()
{
	GLfloat direction[4];
	direction[0] = dir.x; direction[1] = dir.y; direction[2] = dir.z;
	direction[3] = 1.0;
	glLightfv(lightType, GL_POSITION, direction);	
	glLightf(lightType, GL_LINEAR_ATTENUATION, 0.00009f);
}

void LumiereDir::setDir(const vR3& _d)
{
	dir = _d;
}
	
const vR3& LumiereDir::getDir() const
{
	return dir;
}


////////////// LUMIERE SPOT ////////////////////////////////

Spot::Spot() : Lumiere(), cutoff(0.), expo(0.), dir()
{}

Spot::Spot(const pR3& _p, Color* coul, const float& _c, const float& _e, const vR3& _d) : Lumiere(_p, coul), cutoff(_c), expo(_e), dir(_d)
{
	glLightf(lightType, GL_SPOT_CUTOFF,    cutoff);
	glLightf(lightType, GL_SPOT_EXPONENT,  expo);
}

Spot::~Spot()
{
}
		
void Spot::init()
{
	// Attention: pas besoin de toucher aux coeffiients d'attenuation:
	// sinon spot "invisible"
	
	GLfloat direction[3];
	GLfloat position[4];
	
	direction[0] = dir.x;
	direction[1] = dir.y;
	direction[2] = dir.z;
	
	position[0]  = pos.x;
	position[1]  = pos.y;
	position[2]  = pos.z;
	position[3]  = 1.;
	
	glLightfv(lightType, GL_POSITION, position);
	glLightfv(lightType, GL_SPOT_DIRECTION, direction);
}

void Spot::setDir(const vR3& _d)
{
	dir = _d;
}

const vR3& Spot::getDir() const
{
	return dir;
}
