// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <motor/material.hh>

using namespace std;

Material::Material()
{
	amb[0] = amb[1] =  0.;
	amb[2] = 0.;
	amb[3] = 1.;
	
	dif[0] = 0.2; dif[1] = 0.8; dif[2] = 0.2;
	dif[3] = 1.;
	
	spe[0] = spe[1] = spe[2] = spe[3] = 1.;
	
	shi = 15.;
	
	emi[0] = emi[1] = emi[2] = 0.;
	emi[3] = 1.;
}

Material::Material(GLfloat* _amb, GLfloat* _dif, GLfloat* _spe, GLfloat _shi, GLfloat* _emi) : shi(_shi)
{
	memcpy(amb, _amb, sizeof(GLfloat)*4);
	memcpy(dif, _dif, sizeof(GLfloat)*4);
	memcpy(spe, _spe, sizeof(GLfloat)*4);
	memcpy(emi, _emi, sizeof(GLfloat)*4);
}

Material::Material(const Material& src):shi(src.shi)
{
	memcpy(amb, src.amb, sizeof(GLfloat)*4);
	memcpy(dif, src.dif, sizeof(GLfloat)*4);
	memcpy(spe, src.spe, sizeof(GLfloat)*4);
	memcpy(emi, src.emi, sizeof(GLfloat)*4);
}

Material::~Material()
{}

Material& Material::operator =(const Material& src)
{
	memcpy(amb, src.amb, sizeof(GLfloat)*4);
	memcpy(dif, src.dif, sizeof(GLfloat)*4);
	memcpy(spe, src.spe, sizeof(GLfloat)*4);
	memcpy(emi, src.emi, sizeof(GLfloat)*4);
	shi=src.shi;
	return *this;
}

void Material::enable() const
{
	glMaterialf(GL_FRONT, GL_SHININESS, shi);
	
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT);
	glColor4fv(amb);
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
	glColor4fv(dif);
	glColorMaterial(GL_FRONT_AND_BACK, GL_SPECULAR);
	glColor4fv(spe);
	glColorMaterial(GL_FRONT_AND_BACK, GL_EMISSION);
	glColor4fv(emi);
}

void Material::disable() const
{
	glDisable(GL_COLOR_MATERIAL);
}

void Material::setAmbiant(const Color& col)
{
	for(int i(0);i<3;i++)
		*(amb+i) = col[i];
	amb[3] = 1.;
}

void Material::setDiffuse(const Color& col)
{
	for(int i(0);i<3;i++)
		*(dif+i) = col[i];
	dif[3] = 1.;
}

void Material::setSpecular(const Color& col)
{
	for(int i(0);i<3;i++)
		*(spe+i) = col[i];
	spe[3] = 1.;
}

void Material::setEmission(const Color& col)
{
	for(int i(0);i<3;i++)
		*(emi+i) = col[i];
	emi[3] = 1.;
}

void Material::setShininess(const GLfloat& param)
{
	shi = param;
}

