// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <motor/objet.hh>


using namespace std;
using namespace mCG;



/* -------------------- Planes methods -------------------- */

double SIGN_OF_X(const vR3& vect)
{
	if(vect.x < 0.)
		return -1.;
	return 1.;
}

double SIGN_OF_Y(const vR3& vect)
{
	if(vect.y < 0.)
		return -1.;
	return 1.;
}
	
double SIGN_OF_Z(const vR3& vect)
{
	if(vect.z < 0.)
		return -1.;
	return 1.;
}

vR3 pythagore_vector(const vR3& v1, const vR3& v2)
{
	return vR3(sqrt(SQ(v1.x)+SQ(v2.x)), sqrt(SQ(v1.y)+SQ(v2.y)), sqrt(SQ(v1.z)+SQ(v2.z)));
}

SPlan::SPlan() : sNormal(vR3::NUL), sConstant(0.)
{}

SPlan::SPlan(const vR3& _n, const double& _d) : sNormal(_n), sConstant(_d)
{}

SPlan::SPlan(const vR3& _n, const pR3& _p) : sNormal(_n), sConstant(-(_n%vR3(_p)))
{}

SPlan::SPlan(const pR3& _p0, const pR3& _p1, const pR3& _p2) : sNormal(vR3(_p1 - _p0)^vR3(_p2 - _p0)), sConstant(-(sNormal%vR3(_p0)))
{}

void SPlan::setNormal(const vR3& _v)
{
	sNormal = _v;
}

void SPlan::setConstant(const double& _d)
{
	sConstant = _d;
}

void SPlan::Normalize()
{
	sNormal.Normalize();
}

vR3 SPlan::getNormal() const
{
	return sNormal;
}

double SPlan::getConstant() const
{
	return sConstant;
}

SPlan::SSide SPlan::getSide(const pR3& _p) const
{
	double dst(getDistanceTo(_p));
	
	if(dst < 0.) return SPlan::S_NEG_SIDE;
	if(dst > 0.) return SPlan::S_POS_SIDE;
	
	return SPlan::S_NO_SIDE;
}

double SPlan::getDistanceTo(const pR3& _p) const
{
	return -sNormal % vR3(_p) - sConstant;
}

pR3 SPlan::getNearestPoint(const pR3& _p) const
{
	return ( _p - sNormal * (sNormal % vR3(_p) - sConstant) ); 
}

/* -------------------- Class Object Methods - Virtual methods don't use SObject objects !! -------------------- */

SObject::SObject() 
{
	density = 1.0;
	sVisible = true;
	sAnimate = true;
	sSup = NULL;
	mass = 1.0;
}

void SObject::setMass(const double& _p)
{
	mass = _p;
}

void SObject::setDensity(const double& _d)
{
	density = _d;
}

void SObject::setV0(const vR3& v)
{
	sV0 = v;
}

void SObject::setVisible(bool _b)
{
	sVisible = _b;
}

void SObject::setSupport(const SPlan& _s)
{
	sSup = new SSupport(_s);
}

void SObject::unsetSupport()
{
	sSup = NULL;
}

void SObject::setAnimated(bool _b)
{
	sAnimate = _b;
}

double SObject::getMass() const
{
	return mass;
}

double SObject::getDensity() const
{
	return density;
}

const pR3& SObject::getMassCenter() const
{
	return mass_center;
}

vR3 SObject::getV0() const
{
	return sV0;
}

bool SObject::isVisible() const
{
	return sVisible;
}

bool SObject::isAnimated() const
{
	return sAnimate;
}

bool SObject::hasSupport() const
{
	return sSup != NULL;
}

vR3 SObject::getSupportDirection() const
{
	return sSup->getDirection();
}

double SObject::getBoundingSphere()
{
	return 0.;
}

void SObject::setMassCenter(const pR3& _p)
{
	mass_center = _p;
}

/* -------------------- Class Point Methods -  -------------------- */

SPoint::SPoint(double _x, double _y, double _z) : SObject(), sCol(1.,1.,1.)
{
	mass_center = pR3(_x,_y,_z);
}

SPoint::SPoint(const double& _x,const double& _y,const double& _z, const Color& c) : SObject(), sCol(c) 
{
	mass_center = pR3(_x,_y,_z);
}	

SPoint::SPoint(const pR3& p) : SObject(), sCol(1.,1.,1.)
{
	mass_center = p;
}

void SPoint::setColor(const double& c1, const double& c2, const double& c3)
{
	sCol = Color(c1,c2,c3);
}

void SPoint::setX(const double& d)
{
	mass_center.x = d;
}

void SPoint::setY(const double& d)
{
	mass_center.y = d;
}

void SPoint::setZ(const double& d)
{
	mass_center.z = d;
}

void SPoint::addX(const double& d)
{
	mass_center.x += d;
}

void SPoint::addY(const double& d)
{
	mass_center.y += d;
}

void SPoint::addZ(const double& d)
{
	mass_center.z += d;
}

void SPoint::setSupport(SSupport& _s)
{
	sSup = &_s;
	/*mass_center = */
}

void SPoint::updatePosition(const pR3& v)
{
	mass_center = pR3(v.x,v.y,v.z);		
}

void SPoint::display() const
{
	if(sVisible)
	{
		glBegin(GL_POINTS);
		glColor3d(sCol.r, sCol.g, sCol.b);
		glVertex3d(mass_center.x, mass_center.y, mass_center.z);
		glEnd();	
	}
}

int SPoint::getType() const
{
	return S_POINT_TYPE;
}

/* -------------------- Polyhedron Class Methods -------------------- */

SPolyhedron::SPolyhedron(const double& _mass, const pR3& _mCenter, const vR3& _v0, const double& _density) : SObject(), sVList(), sTList(), sFList(), sMode(S_WIRE)
{
	mass = _mass;
	mass_center = _mCenter;
	sV0 = _v0;
	density = _density;
}


SPolyhedron::SPolyhedron(DisplayMode _mode) :  SObject(), sVList(), sTList(), sFList(), sMode(_mode)
{}

void SPolyhedron::setDisplayMode(DisplayMode _mode)
{
	sMode = _mode;
}

SPolyhedron::DisplayMode SPolyhedron::getDisplayMode() const
{
	return sMode;
}

void SPolyhedron::display() const
{
	/* -- Only points for the moment... -- */
	list<SPoint>::const_iterator it = sVList.begin();
	for(;it != sVList.end();it++)
		(*it).display();
}

void SPolyhedron::updatePosition(const pR3& p)
{
	pR3 dpl(mass_center - p);
	
	list<SPoint>::iterator it = sVList.begin();
	for(;it != sVList.end();it++)
		(*it).updatePosition((*it).getMassCenter() - dpl);
	
	mass_center = pR3(p.x,p.y,p.z);
}

void SPolyhedron::setSupport(SSupport& _s)
{
	sSup = &_s;
}

void SPolyhedron::add_point(const pR3& to_add)
{
	sVList.push_back(to_add);
}

int SPolyhedron::getType() const
{
	return 0;
}

void SPolyhedron::rotateX(const double& ang)
{
	Transform mat_trans( 1., 0., 0.,
			     0., cos(ang), sin(ang),
			     0., -sin(ang), cos(ang),
			     0., 0., 0. 
			);
	
	for(list<SPoint>::iterator it=sVList.begin();it!=sVList.end();it++)
		(*it) = SPoint(mat_trans * (*it).getMassCenter());
	
	mass_center = mat_trans * mass_center;	   
}

void SPolyhedron::rotateY(const double& ang)
{
	Transform mat_trans( cos(ang), 0., -sin(ang),
			     0., 1., 0.,
			     sin(ang), 0., cos(ang),
			     0., 0., 0.
			);
	
	for(list<SPoint>::iterator it=sVList.begin();it!=sVList.end();it++)
		(*it) = SPoint(mat_trans * (*it).getMassCenter());
	
	mass_center = mat_trans * mass_center;	  
}

void SPolyhedron::rotateZ(const double& ang)
{
	Transform mat_trans(cos(ang), sin(ang), 0., 
		      	    -sin(ang), cos(ang), 0., 
		      	    0., 0., 1.,
		      	    0., 0., 0.);
	
	for(list<SPoint>::iterator it=sVList.begin();it!=sVList.end();it++)
		(*it) = SPoint(mat_trans *(*it).getMassCenter());
	
	mass_center = mat_trans * mass_center;	  
}


void SPolyhedron::applyTransform(const Transform& mat_trans)
{
	for(list<SPoint>::iterator it=sVList.begin();it!=sVList.end();it++)
		(*it) = SPoint(mat_trans * (*it).getMassCenter());
	
	mass_center = mat_trans * mass_center;
}

/* -------------------- Class Line Methods -------------------- */

SLine::SLine() : sOrigin(), sDirection()
{}

SLine::SLine(const pR3& _orig) : sOrigin(_orig), sDirection()
{}

SLine::SLine(const pR3& _orig, const vR3& _dir) : sOrigin(_orig), sDirection(_dir)
{}

void SLine::setOrigin(const pR3& _o)
{
	sOrigin = _o;
}

void SLine::setDirection(const vR3& _d)
{
	sDirection = _d;
}

const pR3& SLine::getOrigin() const
{
	return sOrigin;
}

const vR3& SLine::getDirection() const
{
	return sDirection;
}

void SLine::display() const
{
	glBegin(GL_LINES);
	glVertex3d(sOrigin.x, sOrigin.y, sOrigin.z);
	glVertex3d(sOrigin.x + sDirection.x, sOrigin.y + sDirection.y, sOrigin.z + sDirection.z);
	glEnd();
}

int SLine::getType() const
{
	return S_LINE_TYPE;
}
/* -------------------- Class Sphere Methods -------------------- */

SSphere::SSphere( double ra, unsigned sl, unsigned st,Color color, const pR3& _c) :  slice(sl), stack(st), rayon(ra)
{
	mass_center = _c;

	double diffg, diffb, gamma, beta;

	//Insertion des sommets
	sVList.insert(sVList.begin(),SPoint(_c.x, _c.y-rayon, _c.z, color));
	diffg = PI/(st-1.0); gamma = (-PI/2.0)+diffg;
	diffb  = 2.*PI/sl;
	
	for(unsigned j(2); j < st; j++)
	{
		beta = 0.;
		for (unsigned i(0); i < sl; i++)
		{
			sVList.insert(sVList.end(),SPoint(_c.x + ra * cos(gamma) * cos(beta),_c.y + ra * sin(gamma),_c.z + ra * cos(gamma) * sin(beta),color));
			beta += diffb;
		}
		gamma += diffg;
	}
	sVList.insert(sVList.end(),SPoint(_c.x, _c.y + rayon, _c.z,color));
}

double SSphere::getRadius() const
{
	return rayon;
}

void SSphere::setSupport(SSupport& _s)
{
	sSup = &_s;
	/*mass_center = */
}

int SSphere::getType() const
{
	return S_SPHERE_TYPE;
}

/* -------------------- Class SCube Methods -------------------- */

SCube::SCube(double _l, double _h, double _w, Color _c, pR3 _p) : sLong(_l), sHeight(_h), sWidth(_w)
{
	// Getting the 8 points
	
	sVList.push_back(SPoint(0., 0., 0., _c));
	sVList.push_back(SPoint(0., -_h, 0., _c));
	sVList.push_back(SPoint(_l, -_h, 0., _c));
	sVList.push_back(SPoint(_l, 0., 0., _c));
	
	sVList.push_back(SPoint(0., -_h, -_w, _c));
	sVList.push_back(SPoint(0., 0., -_w, _c));
	sVList.push_back(SPoint(_l, -_h, -_w, _c));
	sVList.push_back(SPoint(_l, 0., -_w, _c));
	
	// Calculate current mass center
	
	mass_center = (pR3(0., 0., 0.) + pR3(_l, -_h, -_w)) / 2.0;
	
	updatePosition(_p);
}

const double& SCube::getLong() const
{
	return sLong;
}

const double& SCube::getHeight() const
{
	return sHeight;
}

const double& SCube::getWidth() const
{
	return sWidth;
}

int SCube::getType() const
{
	return S_CUBE_TYPE;
}

/* -------------------- Class SBox Methods -------------------- */

const vR3& SBox::getAxis(int i) const
{
	return sAxis[i];
}

vR3* SBox::getAllAxis()
{
	return sAxis;
}

const vR3* SBox::getAllAxis() const
{
	return sAxis;
}

const double& SBox::getExtend(int i) const
{
	return sExtend[i];
}

double* SBox::getAllExtend()
{
	return sExtend;
}

const double* SBox::getAllExtend() const
{
	return sExtend;
}
	
int SBox::getType() const
{
	return S_BOX_TYPE;
}

void SBox::rotateX(const double& ang)
{
	SPolyhedron::rotateX(ang);
	Transform mat_trans( 1., 0., 0.,
			     0., cos(ang), sin(ang),
			     0., -sin(ang), cos(ang),
			     0., 0., 0.
			);
	
	sAxis[0] = mat_trans * sAxis[0];
	sAxis[1] = mat_trans * sAxis[1];
	sAxis[2] = mat_trans * sAxis[2];
}

void SBox::rotateY(const double& ang)
{
	SPolyhedron::rotateY(ang);
	Transform mat_trans( cos(ang), 0., -sin(ang),
			     0., 1., 0.,
			     sin(ang), 0., cos(ang),
			     0., 0., 0.
			);
	
	sAxis[0] = mat_trans * sAxis[0];
	sAxis[1] = mat_trans * sAxis[1];
	sAxis[2] = mat_trans * sAxis[2];
}

void SBox::rotateZ(const double& ang)
{
	SPolyhedron::rotateZ(ang);
	
	Transform mat_trans( cos(ang), sin(ang), 0., 
		      	     -sin(ang), cos(ang), 0., 
		      	     0., 0., 1.,
		      	     0., 0., 0.
			);
	
	sAxis[0] = mat_trans * sAxis[0];
	sAxis[1] = mat_trans * sAxis[1];
	sAxis[2] = mat_trans * sAxis[2];
}

void SBox::applyTransform(const Transform& mat_trans)
{
	SPolyhedron::applyTransform(mat_trans);
	
	sAxis[0] = mat_trans * sAxis[0];
	sAxis[1] = mat_trans * sAxis[1];
	sAxis[2] = mat_trans * sAxis[2];
}

