// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */

#include <motor/particle.hh>

using namespace std;

Particule::Particule() : active(false), life(1.0), fade(0.), color(0., 0., 0.), pos(0., 0., 0.), speed(0., 0., 0.), gravity(0., 0., 0.)
{
//	tex = NULL;
	explo = false;
}

Particule::Particule(const GLfloat& _life, const GLfloat& _fade, const char* tex_name, const Color& col) 
		: active(true), life(_life), fade(_fade), color(col), pos(0., 0., 0.), speed(0., 0., 0.), gravity(0., 0., 0.)
{
//	tex = NULL;
	explo = false;
//	if(tex_name)
//		tex = new TextureGL(tex_name);
	loadGLTextures();
}

Particule::~Particule()
{
//	tex = NULL;
}
		
void Particule::setActive(bool act)
{
	active = act;
}

void Particule::setLife(const GLfloat& l)
{
	life = l;
}

void Particule::setFade(const GLfloat& f)
{
	fade = f;
}

void Particule::setPos(const pR3& p)
{
	pos = p;
}

void Particule::setSpeed(const vR3& s)
{
	speed = s;
}

void Particule::setGravity(const vR3& g)
{
	gravity = g;
}

bool Particule::isActive() const
{
	return active;
}

const GLfloat& Particule::getLife() const
{
	return life;
}

const GLfloat& Particule::getFade() const
{
	return fade;
}

const pR3& Particule::getPos() const
{
	return pos;
}

const vR3& Particule::getSpeed() const
{
	return speed;
}

const vR3& Particule::getGravity() const
{
	return gravity;
}

void Particule::display() 
{
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
    	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, num);
	
	double x, y, z;
	
	x = pos.x;
       	y = pos.y;
        z = pos.z - 10.;
	
	glPushMatrix();
	
	glColor4f(color.r, color.g, color.b, life);
        glBegin(GL_TRIANGLE_STRIP);
                glTexCoord2d(1, 1);
                glVertex3f(x + 0.5f, y + 0.5f, z);
                glTexCoord2d(0, 1);
                glVertex3f(x - 0.5f, y + 0.5f, z);
                glTexCoord2d(1, 0);
                glVertex3f(x + 0.5f, y - 0.5f, z);
                glTexCoord2d(0, 0);
                glVertex3f(x - 0.5f, y - 0.5f, z);
        glEnd();
	
	glPopMatrix();
		
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);

	glBlendFunc(GL_ONE, GL_ZERO);
}

void Particule::explode()
{
	pos.x = 0.0f;
        pos.y = 0.0f;
        pos.z = 0.0f;
        speed.x = (float) ((rand() % 52) - 26.0f) * 10;
        speed.y = (float) ((rand() % 50) - 25.0f) * 10;
        speed.z = (float) ((rand() % 50) - 25.0f) * 10;
}

/*void Particule::setTexture(const char* fic)
{
	if(tex)
		delete tex;
	
	tex = new TextureGL(fic);
}*/

void Particule::loadGLTextures() 
{
/*    textureImage *texti;
    texti = (textureImage *)malloc(sizeof(textureImage));
    */
   /* if(loadBMP("./Textures/Particle.bmp", texti))
    {
*/	   
        glGenTextures(1, &num);
        glBindTexture(GL_TEXTURE_2D, num);
        
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D, 0, 3, textim_pt->width, textim_pt->height, 0, GL_RGB, GL_UNSIGNED_BYTE, textim_pt->data);
/*    }

    if (texti)
    {
        if (texti->data)
            free(texti->data);
        free(texti);
    }
    */
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, num);
}

void Particule::update(const double& factor, const Color& new_color)
{
 	pos.x 	+= speed.x / (factor * 1000);
   	pos.y 	+= speed.y / (factor * 1000);
        pos.z 	+= speed.z / (factor * 1000);
	
        speed.x += gravity.x;
        speed.y += gravity.y;
	speed.z += gravity.z;
	
        life 	-= fade;
	
        if (life < 0.0f)
        {
            life = 1.0f;
            fade = (float) (rand() % 100) / 1000 + 0.003f;
            pos.x = 0.0f;
            pos.y = 0.0f;
            pos.z = 0.0f;
            speed.x = (float) (rand() % 60) - 32.0f;
            speed.y = (float) (rand() % 60) - 30.0f;
            speed.z = (float) (rand() % 60) - 30.0f;
        }
	
	
}
