// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */

#include <algorithm>


#include <motor/scene.hh>


Scene::Scene() : keyIsPress(false), lLum(), lSol(), lObj(), lEnt(), lArm2Pick(), lArm(), lInt(), lAnim()
{
	camera_main  = new CameraFP();
	gui 	     = new HUD(camera_main);
	lBalleAllie  = new AllBullet();
	lBalleEnemy  = new AllBullet();
	interact_cam = new SInteraction((SObject**)(&camera_main->box));
	monstre      = NULL;
	interact_monstre = NULL;
	intel	     = NULL;
	
	interact_cam->init();
	
		
	ALfloat ori[6]={0.f, 1.f ,0.f, 0.f, 1.f, 0.f};	
	alListener3f(AL_POSITION,camera_main->Oeil.x,camera_main->Oeil.y,camera_main->Oeil.z);
	alListener3f(AL_VELOCITY,0.f,0.f,0.f);
	alListenerfv(AL_ORIENTATION,ori);
	
	ttt=new Sound_Table();
	dddd=new Sound(FOOT1,ttt);
	dddd->setPos(camera_main->Oeil);
}

/* Protected constructor: don't need to use it */

Scene::Scene(const Scene& src) 
		: keyIsPress(false), lLum(src.lLum), lObj(src.lObj), lEnt(src.lEnt), lArm2Pick(src.lArm2Pick), lArm(src.lArm), lInt(src.lInt), lAnim(src.lAnim)
{
	camera_main  = src.camera_main;
	gui 	     = src.gui;
	lBalleAllie  = NULL;
	lBalleEnemy  = NULL;
	interact_cam = src.interact_cam;
	monstre      = src.monstre;
	interact_monstre = src.interact_monstre;
	intel	     = src.intel;
	
	ALfloat ori[6]={0.f, 1.f ,0.f, 0.f, 1.f, 0.f};	
	alListener3f(AL_POSITION,camera_main->Oeil.x,camera_main->Oeil.y,camera_main->Oeil.z-0.3f);
	alListener3f(AL_VELOCITY,0.f,0.f,0.f);
	alListenerfv(AL_ORIENTATION,ori);
	
	ttt=new Sound_Table();
	dddd=new Sound(FOOT1,ttt);
	dddd->setPos(camera_main->Oeil);
	
}

/* Protected operator: don't need to use it */

Scene& Scene::operator =(const Scene& src)
{
	camera_main = src.camera_main;
	gui 	    = src.gui;
	lBalleAllie = src.lBalleAllie;
	lBalleEnemy = src.lBalleEnemy;
	lLum 	    = src.lLum;
	lObj 	    = src.lObj;
	lEnt 	    = src.lEnt;
	lArm2Pick   = src.lArm2Pick;
	lArm 	    = src.lArm;
	lInt 	    = src.lInt;
	lAnim 	    = src.lAnim;
	
	interact_cam = src.interact_cam;
	monstre      = src.monstre;
	interact_monstre = src.interact_monstre;
	intel        = src.intel;
	
	return (*this);
}

Scene::~Scene()
{
	// TO DO

	delete dddd;
	delete ttt;

	delete gui ;
	delete camera_main;
	delete lBalleAllie ;
	delete lBalleEnemy ;
	delete interact_cam ;
}

void Scene::initGL()
{
	glClearColor(0., 0., 0., 0.);
	glShadeModel(GL_SMOOTH);
	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	glEnable(GL_BLEND);
	
    	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// On embellit la scene
    	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
			
	glEnable(GL_TEXTURE_2D);
	glEnableClientState( GL_VERTEX_ARRAY );
	glEnableClientState( GL_NORMAL_ARRAY );
}

void Scene::initInteract()
{
	for(list<SInteraction*>::iterator it=lInt.begin();it!=lInt.end();it++)
		(*it)->init();
}

void Scene::initLumiere()
{
	for(list<Lumiere*>::iterator it=lLum.begin();it!=lLum.end();it++)
		(*it)->init();
}

void Scene::addLumiere(Lumiere* _lum)
{
	lLum.push_back(_lum);
}

void Scene::addSol(SObject* _sol)
{
	lSol.push_back(_sol);
}

void Scene::addObject(SObject* _obj, bool interact)
{
	lObj.push_back(_obj);
	if(interact)
		lInt.push_back(new SInteraction(&(lObj.back())));
}

void Scene::addEntity(SEntity* _ent, bool interact)
{
	lEnt.push_back(_ent);
	lObj.push_back(_ent->boundingBox());
	if(interact)
	{
		lInt.push_back(new SInteraction(&(lEnt.back())));
		lInt.back()->init();
	}
}

void Scene::addArme2Pick(SArme* _ent)
{
	lArm2Pick.push_back(_ent);
}

void Scene::addArme(SArme* _arm, bool interact)
{
	gui->addObject(_arm);
}

void Scene::addMonstre(SMonstre* mstr)
{
	monstre = mstr;
	interact_monstre = new SInteraction(monstre->getEntityPt());
	interact_monstre->init();
	intel = new AI(monstre->getEntityPt());
	lObj.push_back(monstre->boundingBox());
}

void Scene::update()
{
	/*static int num_frame = 0;
	bool intersect(false);
	static const double rad2deg = 180. / PI;
	static double spin_jb1 	= 0.;
	static double spin_jb2 	= 0.;
	static double accum    	= 2.0;
	static int mouvement 	= ALLE;
	static double spin_gen1 = 0.;
	static double spin_gen2 = 0.;

	pR3 decalage(interact_monstre->getCenter());
	vR3 v1  = interact_monstre->getSpeed();
	vR3 v2  = monstre->getLookAt();
	decalage.y += monstre->boundingBox()->getHeight() + 0.5;
	num_frame++;
	
	v1.y = 0.;
	v2.y = 0.;
	
	// Si contact (mur, objets, etc ...), le monstre change de direction ==> son regard galement
	
	v1 = v1.Normalize();
	v2 = v2.Normalize();
	
	double direction(acos(v1%v2)*rad2deg);
	
	if(monstre)
	{
		glPushMatrix();
			glTranslatef(decalage.x, decalage.y, decalage.z);
			glRotated(direction, 0., 1., 0.);
			glPushMatrix();
				glRotatef(spin_jb1, 0., 0., 1.);
				for(int i(31);i<36;i++)
					monstre->displayObjNum(i);

			glPopMatrix();

			glPushMatrix();
				glRotatef(-spin_jb1, 0., 0., 1.);
				for(int i(36);i<41;i++)
					monstre->displayObjNum(i);
			glPopMatrix();

			glPushMatrix();			
				for(int i(0);i<31;i++)
					monstre->displayObjNum(i);
				for(int i(41);i<66;i++)
					if(i != 48)
						monstre->displayObjNum(i);
			glPopMatrix();
		glPopMatrix();
		
		interact_monstre->onAction(lSol, intersect, false);
		intersect = false;
		interact_monstre->onAction(lObj, intersect, false);
		if(num_frame > 15) { intel->takeDecision(interact_monstre->getSpeed(), intersect); num_frame=0; }
		interact_monstre->init();
		
		if(spin_jb1 > 20.) mouvement = RETOUR; if(spin_jb1 < -20.) mouvement = ALLE;
		if(mouvement == ALLE)
		{
			spin_jb1  += accum;
			spin_gen2 = 0.;
			if(spin_jb1 <= 10.)
			      spin_gen1 -= accum*1.5;
			else
				if(spin_gen1 <= 0.)
				      spin_gen1 += accum*4.0;  
		} 
		else
		{
			if(mouvement == RETOUR)
			{
			      spin_jb1 -= accum;
			      spin_gen1 = 0.;
			      if(spin_jb1 >= -10.)
				      spin_gen2 -= accum*1.5;
			      else
				if(spin_gen2 <= 0.)
				      spin_gen2 += accum*4.0;  
			} 
		}
	} 
	
	list<SObject*> lMBIS;
	lMBIS.push_back(monstre->boundingBox());
	unsigned short ret=lBalleAllie->onAction(lMBIS);
	if(ret)
	{
        	cout << "BALLE TOUCHE MONSTRE !!! " << endl;
        	monstre->Touched(20.);
    	}
	*/
	bool rien;
	
	for(list<SInteraction*>::iterator it=lInt.begin();it!=lInt.end();it++)
	{
		(*it)->onAction(lObj, rien);
		(*it)->onAction(lSol, rien);
	}

	glutPostRedisplay();
}

void Scene::display()
{
	gui->updatePosition();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	if(keyIsPress) { camera_main->keyPressed(); keyIsPress = false; } else camera_main->keyNotPressed();
	camera_main->Display();

	glPushMatrix();
	
	lBalleAllie->display();
	
	for(list<Lumiere*>::iterator it=lLum.begin();it!=lLum.end();it++)
	{
		(*it)->enable();
		(*it)->init();
	}
	
	for(list<SObject*>::iterator it=lSol.begin();it!=lSol.end();it++)
		(*it)->display();
	for(list<SObject*>::iterator it=lObj.begin();it!=lObj.end();it++)
		(*it)->display();
	for(list<SEntity*>::iterator it=lEnt.begin();it!=lEnt.end();it++)
		(*it)->display();
	for(list<SArme*>::iterator it=lArm2Pick.begin();it!=lArm2Pick.end();it++)
		(*it)->display();
	
	SEntity* to_clean = NULL;
	for(list<Animation*>::iterator it=lAnim.begin();it!=lAnim.end();it++) {
		to_clean = (*it)->display();
		if(to_clean) {
				
			for(list<SInteraction*>::iterator it4=lInt.begin();it4!=lInt.end();it4++)
				if((*((*it4)->sObj)) == to_clean->boundingBox())
				{
					delete (*it4);
					(*it4) = NULL;
					it4 = lInt.erase(it4);
				}
					
			
			for(list<SObject*>::iterator it3=lObj.begin();it3!=lObj.end();it3++)
				if((*it3) == to_clean->boundingBox())
				{
					delete (*it3);
					(*it3) = NULL;
					it3 = lObj.erase(it3);
				}
				
			
			for(list<SEntity*>::iterator it2=lEnt.begin();it2!=lEnt.end();it2++)
				if((*it2) == to_clean)
				{
					it2 = lEnt.erase(it2);
					break;
				}
			
			delete (*it);
			(*it) = NULL;
			it = lAnim.erase(it);
		}					
	}
	
	gui->display();	
	
	glPopMatrix();
	
	update();
	
	for(list<Lumiere*>::iterator it=lLum.begin();it!=lLum.end();it++)
		(*it)->disable();	
}

void Scene::mouseMoveManager(int button, int state, int x, int y)
{
	camera_main->Mouse(button, state, x, y);
	glutPostRedisplay();
}

void Scene::mouseManager(int x, int y)
{
	camera_main->Motion(x,y);
	gui->updatePosition();
	glutPostRedisplay();
}

void Scene::keyManager(unsigned char key, int x, int y, Model* gren_mod)
{
	bool contact(false);
	vR3 nn;	
	vR3 lumDir;
	SArme* arm = NULL;
	SArme* lancerGren = NULL;
	
	switch(key)
	{
		case '6': case '4': case '8': case '2': interact_cam->onAction(lSol, contact);
		
							if(!contact)
							{ 
								camera_main->Keyboard(key, x, y);
								interact_cam->init();
								contact = false;
								interact_cam->onAction(lObj, contact);
								ALfloat x2=camera_main->Oeil.x;
								ALfloat y2=camera_main->Oeil.y;
								ALfloat z2=camera_main->Oeil.z;
								alListener3f(AL_POSITION,x2,y2,z2);
								//alListener3f(AL_ORIENTATION,camera_main->Viser.x,camera_main->Viser.y , camera_main->Viser.z);
								dddd->setPos(camera_main->Oeil);
								dddd->PlaySound();
								if(contact)
								{
									if(key == '8')
									{
										camera_main->Keyboard('2', x, y);
									}
									if(key == '2')
									{
										camera_main->Keyboard('8', x, y);
										camera_main->Keyboard('8', x, y);
									}
									if(key == '4')
									{
										camera_main->Keyboard('6', x, y);
									}
									if(key == '6')
									{
										camera_main->Keyboard('4', x, y);
									}
								}
							}
							else
							{
								if(key == '8')
								{
									camera_main->Keyboard('2', x, y);
								}
								if(key == '2')
								{
									camera_main->Keyboard('8', x, y);
									camera_main->Keyboard('8', x, y);
								}
								if(key == '4')
								{
									camera_main->Keyboard('6', x, y);
								}
								if(key == '6')
								{
									camera_main->Keyboard('4', x, y);
								}
							}
							interact_cam->init();
						
							for(list<SArme*>::iterator it=lArm2Pick.begin();it!=lArm2Pick.end();it++)
							{
								if(Box2BoxCollision(camera_main->box, (*it)->boundingBox()))
								{
									(*it)->resize(0.85);
									(*it)->setInitPos(pR3(0., 0., 0.));
									arm = (*it);
									it = lArm2Pick.erase(it);
									gui->addObject(arm);
									/*glDisable(GL_LIGHTING);
									gui->writeMsg("You pick up the grenades !");
									glEnable(GL_LIGHTING);*/
								}
							}
							break;
							
		case '0':  	gui->setAction(true);
		 		if(!(gui->getArmeType() == SArme::GRENADE))
				{
					nn = camera_main->Viser;
					nn *= 2000.;
					lBalleAllie->addBullet();
					lBalleAllie->updatePosition(camera_main->Oeil, nn);
				}
				else
				{
					nn = vR3(camera_main->Viser.x, camera_main->Viser.y *1.5, camera_main->Viser.z);
					nn *= 60.;
					
					lancerGren = new SArme(SArme::GRENADE);
					lancerGren->loadFromMyModel(*gren_mod);
					lancerGren->resize(2.5);
					lancerGren->rotateX(M_PI);
					lancerGren->setInitPos(camera_main->Oeil);

					lancerGren->setV0(nn);
					lancerGren->setMass(5.);
					lancerGren->setDensity(0.15);
					lancerGren->setDiffuseMaterial(0, Color(0.80, 0.80, 0.80));
					lancerGren->setSpecularMaterial(0, Color(0.0, 0.0, 0.0));
					lancerGren->setDiffuseMaterial(1, Color(0.80, 0.80, 0.80));
					lancerGren->setSpecularMaterial(1, Color(0.0, 0.0, 0.0));
					lancerGren->setDiffuseMaterial(2, Color(0.0, 0.15, 0.0));
					lancerGren->setSpecularMaterial(2, Color(0.0, 0.0, 0.0));
					lancerGren->setDiffuseMaterial(3, Color(0.80, 0.80, 0.80));
					
					addEntity(lancerGren, true);
					
					lAnim.push_back(new AnimGrenScene(lancerGren));
					//glutTimerFunc(3000, activate_grenade, 0);
				}
				break;
		
		case 'c': gui->setActif( -1 ); break;
		case 'h': gui->setHelpActif(!gui->getHelpActif()); break;
		case 'r': gui->Reload(); break;
		case 'f': glutFullScreen(); break;
		case 'a': glutReshapeWindow(800, 600); break;
		case 'b': glutReshapeWindow(1024, 768); break;
		case 27 : 
			if(gui)
				delete gui ;
			if(camera_main)
				delete camera_main;
			if(lBalleAllie)
				delete lBalleAllie ;
			if(lBalleEnemy)
				delete lBalleEnemy ;
			if(interact_cam)
				delete interact_cam ;
			if(dddd)
				delete dddd;
			if(ttt)
				delete ttt;
			if(monstre)
				delete monstre;
			if(interact_monstre)
				delete interact_monstre;
			if(intel)
				delete intel;
			//clean_texture();
			//clean_model();
			//clean_arme();
			alutExit();	
			exit(0);
			break;
			
		default: break;
	}
	glutPostRedisplay();
}

/*
void Scene::init_arme()
{
	BROWNING_Amo	= new SAmo(SAmo::NINE_MM);
	MP40_Amo	= BROWNING_Amo;
	AK47_Amo	= new SAmo(SAmo::P50);
	GRENADE_Amo 	= new SAmo(SAmo::GRENADE);
}

void Scene::clean_arme()
{
	delete GRENADE_Amo;
	delete AK47_Amo;
	delete MP40_Amo;
	delete BROWNING_Amo;
}

void Scene::init_model()
{
	chaise1	 = new Model("./models/chaise1.3ds");
	table1	 = new Model("./models/table1.3ds");
	monstre1 = new Model("./models/Marine.3ds");
	gun1	 = new Model("./models/gun_dep.3ds");
	lampe1	 = new Model("./models/lp_fix.3ds");
	grenade1 = new Model("./models/grenade.3ds"); 
	
	mod_md2  = new Md2Model();
	
	if(!mod_md2->loadModel("./models/monstre_1.md2"))
		cout << "ERROR WHILE LOADING MODELS" << endl;
}


void Scene::clean_model()
{
	delete mod_md2;
	delete grenade1;
	delete lampe1;
	delete gun1;
	delete monstre1;
	delete table1;
	delete chaise1;
}


void Scene::init_texture()
{
	tex_name=new TextureGL("./Textures/143.ppm");		     
	tex_port=new TextureGL("./Textures/porte1.ppm");	     
	tex_gril=new TextureGL("./Textures/grille_air2.ppm");
	tex_sol=new TextureGL("./Textures/sol2.ppm");
	tex_plafon=new TextureGL("./Textures/plafond.ppm");
}

void Scene::clean_texture()
{
	delete tex_plafon;
	delete tex_sol;
	delete tex_gril;
	delete tex_port;
	delete tex_name;
}
*/
