// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#include <iostream>


#include <motor/sound.hh>

using namespace std;

Sound_Table::Sound_Table()
{
	for(int i=0;i<NB_SOUND;i++)
		*(compteur+i)=0;
}

Sound_Table::~Sound_Table()
{
	for(int i=0;i<NB_SOUND;i++)
	{
		*(compteur+i)=0;
		alDeleteBuffers(1,(BUFFER+i));
	}
}


void Sound_Table::LoadSound(const SOUND_TYPE& s)throw (Sound_Exception)
{
	if( *(compteur+s) == 0 )
	{
		alGetError();
		*(BUFFER+s) = alutCreateBufferFromFile( *(SOUND_FILE+s) );
		if( alGetError() != AL_NO_ERROR )
			throw Sound_Exception(Sound_Exception::LOAD_SOUND);
		
	}	
	(*(compteur+s))++;
}

void Sound_Table::UnloadSound(const SOUND_TYPE& s)
{
	if( *(compteur+s) == 1)
	{
		alGetError();
		alDeleteBuffers(1,(BUFFER+s));
		(*(compteur+s))--;
		if( alGetError() != AL_NO_ERROR )
			throw Sound_Exception(Sound_Exception::UNLOAD_SOUND);
	}
	if( *(compteur+s) > 1)
		(*(compteur+s))--;
}

const ALuint& Sound_Table::getBuffer(const SOUND_TYPE& s)const  throw (Sound_Exception)
{
	if( *(compteur+s) >= 1)
		return (*(BUFFER+s));
	else
		throw Sound_Exception(Sound_Exception::GET_BUFFER);	
	//return AL_NONE;
	
}





Sound::Sound(const SOUND_TYPE& s,Sound_Table *t)throw (Sound_Exception) :sound(s) 
{
	table=t;
	table->LoadSound(sound);
	ALuint buf=table->getBuffer(sound);
	alGetError();
	alGenSources(1,&source);
	if( alGetError() != AL_NO_ERROR )
		throw Sound_Exception(Sound_Exception::GEN_SOURCE);
			
	alGetError();		
	alSourcei(source,AL_BUFFER,buf);
	if( alGetError() != AL_NO_ERROR )
		cout<<"Err de alSourcei"<<endl;
		
	alSourcei(source,AL_LOOPING,AL_FALSE);
}


Sound::~Sound()
{
	alSourceStop(source);
	alDeleteSources(1,&source);
	table->UnloadSound(sound);
}

void Sound::PlaySound(bool wait)const throw (Sound_Exception)
{
	if(wait)
	{
		ALint etat;
		alGetSourcei(source,AL_SOURCE_STATE,&etat);
		if( etat != AL_PLAYING )
		{
			alGetError();
			alSourcePlay(source);
			if( alGetError() != AL_NO_ERROR )
				throw Sound_Exception(Sound_Exception::PLAY_SOUND);
		}
	}
	else
	{
		alGetError();
		alSourcePlay(source);
		if( alGetError() != AL_NO_ERROR )
			throw Sound_Exception(Sound_Exception::PLAY_SOUND);
	}
}


void Sound::StopSound()const throw (Sound_Exception)
{	alSourceStop(source);  }
	
void Sound::setPos(const pR3& p)const throw (Sound_Exception)
{
	alGetError();
	alSource3f(source,AL_POSITION,p.x,p.y,p.z);
	if( alGetError() != AL_NO_ERROR )
		throw Sound_Exception(Sound_Exception::SET_POS);	
}

void Sound::setPos(const ALfloat& x,const ALfloat& y,const ALfloat& z)const throw (Sound_Exception)
{	
	alGetError();
	alSource3f(source,AL_POSITION,x,y,z);
	if( alGetError() != AL_NO_ERROR )
		throw Sound_Exception(Sound_Exception::SET_POS);
}


void Sound::setVitesse(const pR3& v)const throw (Sound_Exception)
{	
	alGetError();
	alSource3f(source,AL_VELOCITY,v.x,v.y,v.z);
	if( alGetError() != AL_NO_ERROR )
		throw Sound_Exception(Sound_Exception::SET_VITESSE);
}

void Sound::setVitesse(const ALfloat& x,const ALfloat& y,const ALfloat& z)const throw (Sound_Exception)
{
	alGetError();
	alSource3f(source,AL_VELOCITY,x,y,z);
	if( alGetError() != AL_NO_ERROR )
		throw Sound_Exception(Sound_Exception::SET_VITESSE);

}



Sound_Exception::Sound_Exception(const SOUND_EXCEPTION_CODE& i):code(i)
{}

Sound_Exception::~Sound_Exception()
{}
	
const int& Sound_Exception::getCode()const
{ return code; }

const char* Sound_Exception::getChar()const
{ return (*(SOUND_ERR+code)); }
